# The content of this file was contributed by leppton
# (http://mail.python.org/pipermail/patches/2006-November/020942.html) to
# ctypes project, under MIT License.

# This example shows how to use ctypes module to read all
# function names from dll export directory

import os
if os.name != "nt":
    raise Exception("Wrong OS")

import ctypes as ctypes  # nopep8
import ctypes.wintypes as wintypes  # nopep8


def convert_cdef_to_pydef(line):
    """convert_cdef_to_pydef(line_from_c_header_file) -> python_tuple_string
    'DWORD  var_name[LENGTH];' -> '("var_name", DWORD*LENGTH)'
    doesn't work for all valid c/c++ declarations"""

    l = line[:line.find(';')].split()
    if len(l) != 2:
        return None
    type_ = l[0]
    name = l[1]
    i = name.find('[')
    if i != -1:
        name, brac = name[:i], name[i:][1:-1]
        return '("%s", %s*%s)' % (name, type_, brac)
    else:
        return '("%s", %s)' % (name, type_)


def convert_cdef_to_structure(cdef, name, data_dict=ctypes.__dict__):
    """convert_cdef_to_structure(struct_definition_from_c_header_file)
    -> python class derived from ctypes.Structure
    limited support for c/c++ syntax"""
    py_str = '[\n'
    for line in cdef.split('\n'):
        field = convert_cdef_to_pydef(line)
        if field is not None:
            py_str += ' ' * 4 + field + ',\n'
    py_str += ']\n'

    pyarr = eval(py_str, data_dict)

    class ret_val(ctypes.Structure):
        _fields_ = pyarr
    ret_val.__name__ = name
    ret_val.__module__ = None
    return ret_val

# struct definitions we need to read dll file export table
winnt = (
    ('IMAGE_DOS_HEADER', """\
    WORD   e_magic;
    WORD   e_cblp;
    WORD   e_cp;
    WORD   e_crlc;
    WORD   e_cparhdr;
    WORD   e_minalloc;
    WORD   e_maxalloc;
    WORD   e_ss;
    WORD   e_sp;
    WORD   e_csum;
    WORD   e_ip;
    WORD   e_cs;
    WORD   e_lfarlc;
    WORD   e_ovno;
    WORD   e_res[4];
    WORD   e_oemid;
    WORD   e_oeminfo;
    WORD   e_res2[10];
    LONG   e_lfanew;
"""),

    ('IMAGE_FILE_HEADER', """\
    WORD    Machine;
    WORD    NumberOfSections;
    DWORD   TimeDateStamp;
    DWORD   PointerToSymbolTable;
    DWORD   NumberOfSymbols;
    WORD    SizeOfOptionalHeader;
    WORD    Characteristics;
"""),

    ('IMAGE_DATA_DIRECTORY', """\
    DWORD   VirtualAddress;
    DWORD   Size;
"""),

    ('IMAGE_OPTIONAL_HEADER32', """\
    WORD    Magic;
    BYTE    MajorLinkerVersion;
    BYTE    MinorLinkerVersion;
    DWORD   SizeOfCode;
    DWORD   SizeOfInitializedData;
    DWORD   SizeOfUninitializedData;
    DWORD   AddressOfEntryPoint;
    DWORD   BaseOfCode;
    DWORD   BaseOfData;
    DWORD   ImageBase;
    DWORD   SectionAlignment;
    DWORD   FileAlignment;
    WORD    MajorOperatingSystemVersion;
    WORD    MinorOperatingSystemVersion;
    WORD    MajorImageVersion;
    WORD    MinorImageVersion;
    WORD    MajorSubsystemVersion;
    WORD    MinorSubsystemVersion;
    DWORD   Win32VersionValue;
    DWORD   SizeOfImage;
    DWORD   SizeOfHeaders;
    DWORD   CheckSum;
    WORD    Subsystem;
    WORD    DllCharacteristics;
    DWORD   SizeOfStackReserve;
    DWORD   SizeOfStackCommit;
    DWORD   SizeOfHeapReserve;
    DWORD   SizeOfHeapCommit;
    DWORD   LoaderFlags;
    DWORD   NumberOfRvaAndSizes;
    IMAGE_DATA_DIRECTORY DataDirectory[IMAGE_NUMBEROF_DIRECTORY_ENTRIES];
""",
     {'IMAGE_NUMBEROF_DIRECTORY_ENTRIES': 16}),

    ('IMAGE_NT_HEADERS', """\
    DWORD Signature;
    IMAGE_FILE_HEADER FileHeader;
    IMAGE_OPTIONAL_HEADER32 OptionalHeader;
"""),

    ('IMAGE_EXPORT_DIRECTORY', """\
    DWORD   Characteristics;
    DWORD   TimeDateStamp;
    WORD    MajorVersion;
    WORD    MinorVersion;
    DWORD   Name;
    DWORD   Base;
    DWORD   NumberOfFunctions;
    DWORD   NumberOfNames;
    DWORD   AddressOfFunctions;
    DWORD   AddressOfNames;
    DWORD   AddressOfNameOrdinals;
"""),
)

# Construct python ctypes.Structures from above definitions
data_dict = dict(wintypes.__dict__)
for definition in winnt:
    name = definition[0]
    def_str = definition[1]
    if len(definition) == 3:
        data_dict.update(definition[2])
    type_ = convert_cdef_to_structure(def_str, name, data_dict)
    data_dict[name] = type_
    globals()[name] = type_

    ptype = ctypes.POINTER(type_)
    pname = 'P' + name
    data_dict[pname] = ptype
    globals()[pname] = ptype

del data_dict
del winnt


class DllException(Exception):
    pass


def read_export_table(dll_name, mmap=False, use_kernel=False):
    """
    read_export_table(dll_name [,mmap=False [,use_kernel=False]]])
     -> list of exported names

    default is to load dll into memory: dll sections are aligned to
    page boundaries, dll entry points is called, etc...

    with mmap=True dll file image is mapped to memory, Relative Virtual
    Addresses (RVAs) must be mapped to real addresses manually

    with use_kernel=True direct kernel32.dll calls are used,
    instead of python mmap module

    see http://www.windowsitlibrary.com/Content/356/11/1.html
    for details on Portable Executable (PE) file format
    """
    if not mmap:
        dll = ctypes.cdll.LoadLibrary(dll_name)
        if dll is None:
            raise DllException("Cant load dll")
        base_addr = dll._handle

    else:
        if not use_kernel:
            fileh = open(dll_name)
            if fileh is None:
                raise DllException("Cant load dll")
            import mmap
            m = mmap.mmap(fileh.fileno(), 0, None, mmap.ACCESS_READ)
            # id(m)+8 sucks, is there better way?
            base_addr = ctypes.cast(id(m) + 8, ctypes.POINTER(ctypes.c_int))[0]
        else:
            kernel32 = ctypes.windll.kernel32
            if kernel32 is None:
                raise DllException("cant load kernel")
            fileh = kernel32.CreateFileA(dll_name, 0x00120089, 1, 0, 3, 0, 0)
            if fileh == 0:
                raise DllException(
                    "Cant open, errcode = %d" %
                    kernel32.GetLastError())
            maph = kernel32.CreateFileMappingW(fileh, 0, 0x8000002, 0, 0, 0)
            if maph == 0:
                raise DllException(
                    "Cant mmap, errocode = %d" %
                    kernel32.GetLastError())
            base_addr = ctypes.windll.kernel32.MapViewOfFile(
                maph, 0x4, 0, 0, 0)
            if base_addr == 0:
                raise DllException(
                    "Cant mmap(2), errocode = %d" %
                    kernel32.GetLastError())

        dbghelp = ctypes.windll.dbghelp
        if dbghelp is None:
            raise DllException("dbghelp.dll not installed")
        pimage_nt_header = dbghelp.ImageNtHeader(base_addr)
        if pimage_nt_header == 0:
            raise DllException("Cant find IMAGE_NT_HEADER")

        # Functions like dbghelp.ImageNtHeader above have no type information
        # let's make one prototype for extra buzz
        # PVOID ImageRvaToVa(PIMAGE_NT_HEADERS NtHeaders, PVOID Base,
        # ULONG Rva, PIMAGE_SECTION_HEADER* LastRvaSection)
        # we use integers instead of pointers, coz integers are better
        # for pointer arithmetic
        prototype = ctypes.WINFUNCTYPE(
            ctypes.c_int,
            ctypes.c_int,
            ctypes.c_int,
            ctypes.c_int,
            ctypes.c_int)
        paramflags = (
            (1, "NtHeaders", pimage_nt_header),
            (1, "Base", base_addr),
            (1, "Rva"),
            (1, "LastRvaSection", 0))
        image_rva_to_va = prototype(('ImageRvaToVa', dbghelp), paramflags)

    def cast_rva(rva, type_):
        va = base_addr + rva
        if mmap and va > pimage_nt_header:
            va = image_rva_to_va(Rva=rva)
            if va == 0:
                raise DllException("ImageRvaToVa failed")
        return ctypes.cast(va, type_)

    if not mmap:
        dos_header = cast_rva(0, PIMAGE_DOS_HEADER)[0]
        if dos_header.e_magic != 0x5A4D:
            raise DllException("IMAGE_DOS_HEADER.e_magic error")
        nt_header = cast_rva(dos_header.e_lfanew, PIMAGE_NT_HEADERS)[0]
    else:
        nt_header = ctypes.cast(pimage_nt_header, PIMAGE_NT_HEADERS)[0]
    if nt_header.Signature != 0x00004550:
        raise DllException("IMAGE_NT_HEADERS.Signature error")

    opt_header = nt_header.OptionalHeader
    if opt_header.Magic != 0x010b:
        raise DllException("IMAGE_OPTIONAL_HEADERS32.Magic error")

    ret_val = []
    exports_dd = opt_header.DataDirectory[0]
    if opt_header.NumberOfRvaAndSizes > 0 or exports_dd != 0:
        export_dir = cast_rva(
            exports_dd.VirtualAddress,
            PIMAGE_EXPORT_DIRECTORY)[0]

        n_names = export_dir.NumberOfNames
        if n_names > 0:
            p_names_type = ctypes.POINTER(ctypes.c_int * n_names)
            names = cast_rva(export_dir.AddressOfNames, p_names_type)[0]
            for rva in names:
                name = cast_rva(rva, ctypes.c_char_p).value
                ret_val.append(name)

    if mmap:
        if use_kernel:
            kernel32.UnmapViewOfFile(base_addr)
            kernel32.CloseHandle(mapH)
            kernel32.CloseHandle(fileh)
        else:
            m.close()
            fileh.close()
    return ret_val


if __name__ == '__main__':
    import sys
    if len(sys.argv) != 2:
        print('usage: %s dll_file_name' % sys.argv[0])
        sys.exit()
    # names = read_export_table(sys.argv[1], mmap=False, use_kernel=False)
    names = read_export_table(sys.argv[1], mmap=False, use_kernel=False)
    for name in names:
        print(name)



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case

from pygccxml import parser
from pygccxml import declarations
from pygccxml.declarations import type_traits


class Test(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = 'unnamed_classes.hpp'
        self.global_ns = None

    def setUp(self):
        if not self.global_ns:
            decls = parser.parse([self.header], self.config)
            self.global_ns = declarations.get_global_namespace(decls)
            self.global_ns.init_optimizer()

    def validate_bitfields(self, parent, bitfields):
        for key in bitfields:
            var = parent.variable(key)
            self.assertEqual(var.bits, bitfields[key])

    def do_union_test(self, union_name, bitfields):
        s2 = self.global_ns.class_('S2')
        self.assertFalse(declarations.is_union(s2))
        self.assertEqual(s2.parent.name, 'S1')
        self.assertFalse(declarations.is_union(s2.parent))

        union = s2.variable(union_name)
        self.assertTrue(declarations.is_union(union.decl_type))

        union_type = type_traits.remove_declarated(union.decl_type)
        self.validate_bitfields(union_type, bitfields)
        self.assertIsNotNone(union_type.variable('raw'))

    def test_union_Flags(self):
        flags_bitfields = {
            'hasItemIdList': 1,
            'pointsToFileOrDir': 1,
            'hasDescription': 1,
            'hasRelativePath': 1,
            'hasWorkingDir': 1,
            'hasCmdLineArgs': 1,
            'hasCustomIcon': 1,
            'useWorkingDir': 1,
            'unused': 24,
        }
        self.do_union_test('flags', flags_bitfields)

    def test_unnamed_unions(self):
        fileattribs_bitfields = {
            'isReadOnly': 1,
            'isHidden': 1,
            'isSystem': 1,
            'isVolumeLabel': 1,
            'isDir': 1,
            'isModified': 1,
            'isEncrypted': 1,
            'isNormal': 1,
            'isTemporary': 1,
            'isSparse': 1,
            'hasReparsePoint': 1,
            'isCompressed': 1,
            'isOffline': 1,
            'unused': 19,
        }
        self.do_union_test('fileattribs', fileattribs_bitfields)

    def test_anonymous_unions(self):
        s3 = self.global_ns.class_('S3')
        self.assertEqual(s3.parent.name, 'S1')

        s3_vars = ['anon_mem_c', 'anon_mem_i', 's3_mem', 's2']
        for var in s3_vars:
            self.assertFalse(declarations.is_union(s3.variable(var).decl_type))


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import os
import unittest
import parser_test_case

from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):

    """
    Test the remove__va_list_tag option

    With CastXML and clang some __va_list_tag declarations are present in the
    tree. This options allows to remove them when parsing the xml file.

    """

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.__code = os.linesep.join(['struct a{};'])
        self.known_typedefs = [
            "__int128_t", "__uint128_t", "__builtin_va_list"]
        self.known_typedefs_llvm39 = \
            self.known_typedefs + ["__builtin_ms_va_list"]
        self.known_classes = ["a", "__va_list_tag"]
        self.known_classes_llvm39 = \
            self.known_classes + ["__NSConstantString_tag"]

    def test_keep_va_list_tag(self):

        if "gccxml" in self.config.xml_generator:
            return True

        self.config.flags = ["f1"]
        src_reader = parser.source_reader_t(self.config)
        decls = declarations.make_flatten(src_reader.read_string(self.__code))

        classes = [
            i for i in decls if isinstance(i, declarations.class_t)]

        typedefs = [
            i for i in decls if isinstance(i, declarations.typedef_t)]

        variables = [
            i for i in decls if isinstance(i, declarations.variable_t)]

        tag = "__va_list_tag"

        self.assertTrue(tag in [class_.name for class_ in classes])
        self.assertTrue("a" in [class_.name for class_ in classes])
        if len(classes) == 2:
            for c in self.known_classes:
                self.assertTrue(c in [cl.name for cl in classes])
        elif len(classes) == 3:
            for c in self.known_classes_llvm39:
                # This is for llvm 3.9
                self.assertTrue(c in [cl.name for cl in classes])

        self.assertTrue(len(typedefs) == 4 or len(typedefs) == 5)
        if len(typedefs) == 5:
            # This is for llvm 3.9. The class __va_list_tag struct is still
            # there but the typedef is gone
            for t in self.known_typedefs_llvm39:
                self.assertTrue(t in [ty.name for ty in typedefs])
            self.assertTrue(
                "__NSConstantString_tag" in
                [class_.name for class_ in classes])
            self.assertTrue(
                "__NSConstantString" in [ty.name for ty in typedefs])
        else:
            for t in self.known_typedefs:
                self.assertTrue(t in [ty.name for ty in typedefs])

        self.assertTrue(
            tag in [var.decl_string.split("::")[1] for var in variables])

        # 4 variables in __va_list_tag, and 4 more in __NSConstantString_tag
        # for llvm 3.9
        self.assertTrue(len(variables) == 4 or len(variables) == 8)

    def test_remove_va_list_tag(self):

        if "gccxml" in self.config.xml_generator:
            return True

        self.config.flags = []
        src_reader = parser.source_reader_t(self.config)
        decls = declarations.make_flatten(src_reader.read_string(self.__code))

        classes = [
            i for i in decls if isinstance(i, declarations.class_t)]

        typedefs = [
            i for i in decls if isinstance(i, declarations.typedef_t)]

        variables = [
            i for i in decls if isinstance(i, declarations.variable_t)]

        tag = "__va_list_tag"

        self.assertFalse(tag in [class_.name for class_ in classes])
        self.assertTrue("a" in [class_.name for class_ in classes])
        self.assertTrue(len(classes) == 1)

        self.assertFalse(tag in [ty.name for ty in typedefs])
        self.assertTrue(len(typedefs) == 3 or len(typedefs) == 4)
        if len(typedefs) == 4:
            # This is for llvm 3.9
            for t in self.known_typedefs_llvm39:
                self.assertTrue(t in [ty.name for ty in typedefs])
            self.assertFalse(
                "__NSConstantString_tag"
                in [class_.name for class_ in classes])
            self.assertFalse(
                "__NSConstantString" in [ty.name for ty in typedefs])
        else:
            for t in self.known_typedefs:
                self.assertTrue(t in [ty.name for ty in typedefs])

        self.assertFalse(
            tag in [var.decl_string.split("::")[1] for var in variables])
        self.assertTrue(len(variables) == 0)


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import warnings
from . import scanner

# keep py2exe happy
import xml.etree.ElementTree

import xml.etree.cElementTree as ElementTree


class etree_saxifier_t(object):

    def __init__(self, etree, handler):
        """
        Deprecated since 1.8.0. Will be removed in 1.9.0.

        """
        warnings.warn("etree_saxifier_t is deprecated.\n", DeprecationWarning)
        self.__root_elem = etree.getroot()
        self.__handler = handler

    def saxify(self):
        self.__handler.startDocument()
        self.__recursive_saxify(self.__root_elem)
        self.__handler.endDocument()

    def __recursive_saxify(self, element):
        self.__handler.startElement(element.tag, element.attrib)
        for e in element:
            self.__recursive_saxify(e)
        self.__handler.endElement(element.tag)


class etree_scanner_t(scanner.scanner_t):

    def __init__(self, xml_file, decl_factory, *args):
        """
        Deprecated since 1.8.0. Will be removed in 1.9.0.

        """
        warnings.warn(
            "etree_scanner_t is deprecated.\n" +
            "Please use ietree_scanner_t instead.", DeprecationWarning)
        scanner.scanner_t.__init__(self, xml_file, decl_factory, *args)

    def read(self):
        tree = ElementTree.parse(self.xml_file)
        saxifier = etree_saxifier_t(tree, self)
        saxifier.saxify()


class ietree_scanner_t(scanner.scanner_t):

    def __init__(self, xml_file, decl_factory, *args):
        scanner.scanner_t.__init__(self, xml_file, decl_factory, *args)

    def read(self):
        context = ElementTree.iterparse(
            self.xml_file,
            events=("start", "end"))
        for event, elem in context:
            if event == 'start':
                self.startElement(elem.tag, elem.attrib)
            else:
                self.endElement(elem.tag)
                elem.clear()
        self.endDocument()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case

from pygccxml import parser
from pygccxml import declarations
from pygccxml import utils


class Test(parser_test_case.parser_test_case_t):
    global_ns = None
    COMPILATION_MODE = parser.COMPILATION_MODE.ALL_AT_ONCE

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = 'declarations_calldef.hpp'
        self.global_ns = None

    def setUp(self):
        if not Test.global_ns:
            decls = parser.parse([self.header], self.config)
            Test.global_ns = declarations.get_global_namespace(decls)
            Test.global_ns.init_optimizer()
        self.global_ns = Test.global_ns

    def test_regex(self):
        criteria = declarations.regex_matcher_t(
            'oper.*',
            lambda decl: decl.name)
        operators = declarations.matcher.find(criteria, self.global_ns)
        operators = [d for d in operators if not d.is_artificial]
        self.assertTrue(6 == len(operators))

    def test_access_type(self):
        criteria = declarations.access_type_matcher_t(
            declarations.ACCESS_TYPES.PUBLIC)
        public_members = declarations.matcher.find(criteria, self.global_ns)
        public_members = [d for d in public_members if not d.is_artificial]
        if "CastXML" in utils.xml_generator:
            nbr = len(public_members)
            self.assertTrue(nbr in [17, 21])
            if nbr == 21:
                # We are using llvm 3.9, see bug #32. Make sure the 4 names
                # are still there
                ll = ["isa", "flags", "str", "length"]
                for l in ll:
                    self.assertTrue(l in [mbr.name for mbr in public_members])
        else:
            self.assertTrue(17 == len(public_members))

    def test_or_matcher(self):
        criteria1 = declarations.regex_matcher_t(
            "oper.*",
            lambda decl: decl.name)
        criteria2 = declarations.access_type_matcher_t(
            declarations.ACCESS_TYPES.PUBLIC)
        found = declarations.matcher.find(
            criteria1 | criteria2,
            self.global_ns)

        if "CastXML" in utils.xml_generator:
            found = [d for d in found if not d.is_artificial]
            self.assertTrue(len(found) != 35)
        elif "0.9" in utils.xml_generator:
            found = [d for d in found if not d.is_artificial]
            self.assertTrue(15 <= len(found) <= 21)
        else:
            self.assertTrue(19 <= len(found) <= 25)

    def test_and_matcher(self):
        criteria1 = declarations.regex_matcher_t(
            'oper.*',
            lambda decl: decl.name)
        criteria2 = declarations.access_type_matcher_t(
            declarations.ACCESS_TYPES.PUBLIC)
        found = declarations.matcher.find(
            criteria1 & criteria2,
            self.global_ns)
        found = [d for d in found if not d.is_artificial]
        self.assertTrue(len(found) <= 6)

    def test_not_matcher(self):
        criteria1 = declarations.regex_matcher_t(
            'oper.*',
            lambda decl: decl.name)
        found = declarations.matcher.find(~(~criteria1), self.global_ns)
        found = [d for d in found if not d.is_artificial]
        self.assertTrue(len(found) == 6)


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

"""
provides low-level functionality, needed to undecorate\demangle compiler
generated unique names and map them to the declarations

On Windows:
    ctypes package is used to call `UnDecorateSymbolName` function from
    `dbghelp.dll`

On Linux:
    "nm" utility is used.
"""

import os
import re
import ctypes
import warnings
from .. import declarations


class UNDECORATE_NAME_OPTIONS(object):

    """defines few constants for `UnDecorateSymbolName` function"""

    UNDNAME_COMPLETE = 0x0000  # Enables full undecoration.
    # Removes leading underscores from Microsoft extended keywords.
    UNDNAME_NO_LEADING_UNDERSCORES = 0x0001
    # Disables expansion of Microsoft extended keywords.
    UNDNAME_NO_MS_KEYWORDS = 0x0002
    # Disables expansion of return type for primary declaration.
    UNDNAME_NO_FUNCTION_RETURNS = 0x0004
    # Disables expansion of the declaration model.
    UNDNAME_NO_ALLOCATION_MODEL = 0x0008
    # Disables expansion of the declaration language specifier.
    UNDNAME_NO_ALLOCATION_LANGUAGE = 0x0010
    UNDNAME_RESERVED1 = 0x0020  # RESERVED.
    UNDNAME_RESERVED2 = 0x0040  # RESERVED.
    UNDNAME_NO_THISTYPE = 0x0060  # Disables all modifiers on the this type.
    # Disables expansion of access specifiers for members.
    UNDNAME_NO_ACCESS_SPECIFIERS = 0x0080
    # Disables expansion of "throw-signatures" for functions and pointers to
    # functions.
    UNDNAME_NO_THROW_SIGNATURES = 0x0100
    # Disables expansion of static or virtual members.
    UNDNAME_NO_MEMBER_TYPE = 0x0200
    # Disables expansion of the Microsoft model for UDT returns.
    UNDNAME_NO_RETURN_UDT_MODEL = 0x0400
    UNDNAME_32_BIT_DECODE = 0x0800  # Undecorates 32-bit decorated names.
    # Gets only the name for primary declaration; returns just [scope::]name.
    # Expands template params.
    UNDNAME_NAME_ONLY = 0x1000
    # Input is just a type encoding; composes an abstract declarator.
    UNDNAME_TYPE_ONLY = 0x2000
    # The real template parameters are available.
    UNDNAME_HAVE_PARAMETERS = 0x4000
    UNDNAME_NO_ECSU = 0x8000  # Suppresses enum/class/struct/union.
    # Suppresses check for valid identifier characters.
    UNDNAME_NO_IDENT_CHAR_CHECK = 0x10000
    UNDNAME_NO_PTR64 = 0x20000  # Does not include ptr64 in output.

    UNDNAME_SCOPES_ONLY = UNDNAME_NO_LEADING_UNDERSCORES \
        | UNDNAME_NO_MS_KEYWORDS \
        | UNDNAME_NO_FUNCTION_RETURNS \
        | UNDNAME_NO_ALLOCATION_MODEL \
        | UNDNAME_NO_ALLOCATION_LANGUAGE \
        | UNDNAME_NO_ACCESS_SPECIFIERS \
        | UNDNAME_NO_THROW_SIGNATURES \
        | UNDNAME_NO_MEMBER_TYPE \
        | UNDNAME_NO_ECSU \
        | UNDNAME_NO_IDENT_CHAR_CHECK

    SHORT_UNIQUE_NAME = UNDNAME_NO_MS_KEYWORDS \
        | UNDNAME_NO_ACCESS_SPECIFIERS | UNDNAME_NO_ECSU


class undname_creator_t(object):

    """implementation details - should not be used directly

    formats declarations string representation and exported symbols, so they
    could be matched later.

    The class formats variables, free and member functions, symbols exported
    from .dll, .map and .so files.

    On Windows, the class works with unique name produced by MSVC compiler and
    with undecorated names produced by `dbghelp.dll`

    On Linux, the class works with mangled names produced by GCC-XML
    ( GCC 4.2 ) compiler and demangled name produced by "nm" utility.
    """

    def __init__(self):

        warnings.warn(
            "undname_creator_t is deprecated.\n" +
            "Please have a look at the changelog for an explanation " +
            "(since 1.8.0)",
            DeprecationWarning)

        if 'nt' == os.name:
            import ctypes.wintypes
            self.__undname = ctypes.windll.dbghelp.UnDecorateSymbolName
            self.__undname.argtypes = [
                ctypes.c_char_p,
                ctypes.c_char_p,
                ctypes.c_uint,
                ctypes.c_uint]

        self.__clean_ecsu = (
            re.compile(r'(?P<startswith>^|\W)(?:(class|enum|struct|union)\s)'),
            '%(startswith)s')
        self.__fundamental_types = (
            ('short unsigned int',
             'unsigned short'),
            ('short int',
             'short'),
            ('long int',
             'long'),
            ('long unsigned int',
             'unsigned long'))
        self.__calling_conventions = (
            re.compile((
                r'(?P<startswith>^|\s)(?:__(cdecl|clrcall|stdcall|fastcall' +
                '|thiscall)\s)')), '%(startswith)s')

    def normalize_undecorated(self, undname, options=None):
        if options is None:
            options = UNDECORATE_NAME_OPTIONS.SHORT_UNIQUE_NAME
        if UNDECORATE_NAME_OPTIONS.UNDNAME_NO_ECSU & options:
            expr, substitute = self.__clean_ecsu
            undname = expr.sub(lambda m: substitute % m.groupdict(), undname)
        if UNDECORATE_NAME_OPTIONS.UNDNAME_NO_ACCESS_SPECIFIERS & options:
            for prefix in ('public: ', 'private: ', 'protected: '):
                if undname.startswith(prefix):
                    undname = undname[len(prefix):]
                    break
        if UNDECORATE_NAME_OPTIONS.UNDNAME_NO_MS_KEYWORDS & options:
            expr, substitute = self.__calling_conventions
            undname = expr.sub(lambda m: substitute % m.groupdict(), undname)
        return undname.strip()

    def undecorate_blob(self, name, options=None):
        if options is None:
            options = UNDECORATE_NAME_OPTIONS.SHORT_UNIQUE_NAME
        buffer = ctypes.create_string_buffer(1024 * 16)
        res = self.__undname(str(name), buffer, ctypes.sizeof(buffer), options)
        if res:
            return self.normalize_undecorated(str(buffer[:res]))
        else:
            return name

    def __remove_leading_scope(self, s):
        if s and s.startswith('::'):
            return s[2:]
        else:
            return s

    def __format_type_as_undecorated(self, type_, is_argument, hint):
        result = []
        type_ = declarations.remove_alias(type_)
        if declarations.is_array(type_):
            result.append(declarations.array_item_type(type_).decl_string)
            result.append('*')
            if is_argument:
                result.append('const')
        else:
            result.append(self.__remove_leading_scope(type_.decl_string))

        result = ' '.join(result)
        if hint == 'nm':
            for x in ('*', '&'):
                result = result.replace(' ' + x, x)
        return result

    def __normalize(self, name):
        for what, with_ in self.__fundamental_types:
            name = name.replace(what, with_)
        name = name.replace(', ', ',')
        return name

    def format_argtypes(self, argtypes, hint):
        if not argtypes:
            if hint == 'msvc':
                return 'void'
            else:
                return ''
        else:
            argsep = ','
            if hint == 'nm':
                # ugly hack, later, I will replace ', ' with ',', so single
                # space will still exist
                argsep = ',  '
            return argsep.join(
                map(lambda type_: self.__format_type_as_undecorated(
                    type_, True, hint), argtypes))

    def format_calldef(self, calldef, hint):
        calldef_type = calldef.function_type()
        result = []
        is_mem_fun = isinstance(calldef, declarations.member_calldef_t)
        if is_mem_fun and hint == 'msvc' and calldef.virtuality != \
                declarations.VIRTUALITY_TYPES.NOT_VIRTUAL:
            result.append('virtual ')
        if is_mem_fun and hint == 'msvc'and calldef.has_static:
            result.append('static ')
        if hint == 'msvc' and calldef_type.return_type:
            # nm doesn't dump return type information
            result.append(
                self.__format_type_as_undecorated(
                    calldef.return_type, False, hint))
            result.append(' ')
        if is_mem_fun:
            result.append(
                self.__remove_leading_scope(
                    calldef.parent.decl_string) + '::')

        result.append(calldef.name)
        if isinstance(
            calldef, (declarations.constructor_t, declarations.destructor_t)) \
           and declarations.templates.is_instantiation(calldef.parent.name):
            if hint == 'msvc':
                result.append('<%s>' % ','.join(
                    declarations.templates.args(calldef.parent.name)))

        result.append('(%s)' % self.format_argtypes(
            calldef_type.arguments_types, hint))
        if is_mem_fun and calldef.has_const:
            if hint == 'nm':
                result.append(' ')
            result.append('const')
        return ''.join(result)

    def format_var(self, decl, hint):
        result = []
        is_mem_var = isinstance(decl.parent, declarations.class_t)
        if is_mem_var and decl.type_qualifiers.has_static and hint == 'msvc':
            result.append('static ')
        if hint == 'msvc':
            result.append(
                self.__format_type_as_undecorated(decl.decl_type, False, hint))
            result.append(' ')
        if is_mem_var:
            result.append(
                self.__remove_leading_scope(decl.parent.decl_string) + '::')
        result.append(decl.name)
        return ''.join(result)

    def format_decl(self, decl, hint=None):
        """returns string, which contains full function name formatted exactly
        as result of `dbghelp.UnDecorateSymbolName`, with
        UNDNAME_NO_MS_KEYWORDS | UNDNAME_NO_ACCESS_SPECIFIERS | UNDNAME_NO_ECSU
        options.

        Different compilers/utilities undecorate/demangle mangled string
        ( unique names ) in a different way.
        `hint` argument will tell pygccxml how to format declarations, so they
        could be mapped later to the blobs.
        The valid options are: "msvc" and "nm".
        """

        if hint is None:
            if 'nt' == os.name:
                hint = 'msvc'
            else:
                hint = 'nm'

        if isinstance(decl, declarations.calldef_t):
            name = self.format_calldef(decl, hint)
        elif isinstance(decl, declarations.variable_t):
            name = self.format_var(decl, hint)
        else:
            raise NotImplementedError()
        return self.__normalize(name)



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case

from pygccxml import parser
from pygccxml import declarations
from pygccxml import utils

code = \
    """
template <typename T> struct A {};
template <int N> struct A<const char[N]>
{ static int size(const char[N]) { return N - 1; } };
"""


class Test(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)

    def test(self):
        """
        The purpose of this test was to check if changes to GCCXML
        would lead to changes in the outputted xml file (Meaning
        the bug was fixed).

        GCCXML wrongly outputted partial template specialization.
        CastXML does not have this bug. In this case we check if
        the template specialization can not be found; which is the
        expected/wanted behaviour.

        https://github.com/CastXML/CastXML/issues/20

        """

        src_reader = parser.source_reader_t(self.config)
        global_ns = declarations.get_global_namespace(
            src_reader.read_string(code))
        if 'GCCXML' in utils.xml_generator:
            a = global_ns.class_('A<const char [N]>')
            a.mem_fun('size')
        elif 'CastXML' in utils.xml_generator:
            self.assertRaises(
                global_ns.declaration_not_found_t,
                lambda: global_ns.class_('A<const char [N]>'))


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import os
import stat
import unittest
import parser_test_case

from pygccxml import utils
from pygccxml import parser


class Test(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.__fname = 'core_types.hpp'

    def test(self):
        fconfig = parser.file_configuration_t(
            data=self.__fname,
            content_type=parser.CONTENT_TYPE.CACHED_SOURCE_FILE)
        try:
            prj_reader = parser.project_reader_t(self.config)
            prj_reader.read_files(
                [fconfig],
                compilation_mode=parser.COMPILATION_MODE.FILE_BY_FILE)
            self.assertTrue(os.path.exists(fconfig.cached_source_file))
            mtime1 = os.stat(fconfig.cached_source_file)[stat.ST_MTIME]
            prj_reader.read_files(
                [fconfig],
                compilation_mode=parser.COMPILATION_MODE.FILE_BY_FILE)
            mtime2 = os.stat(fconfig.cached_source_file)[stat.ST_MTIME]
            self.assertTrue(mtime1 == mtime2)
        finally:
            utils.remove_file_no_raise(fconfig.cached_source_file, self.config)


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case
from pygccxml import parser
from pygccxml import declarations
from pygccxml import utils


class Test(parser_test_case.parser_test_case_t):
    global_ns = None

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = 'remove_template_defaults.hpp'

    def setUp(self):
        if not Test.global_ns:
            decls = parser.parse([self.header], self.config)
            Test.global_ns = declarations.get_global_namespace(decls)
            Test.global_ns.init_optimizer()

    def test_vector(self):
        v_int = self.global_ns.typedef('v_int')
        v_traits = declarations.vector_traits
        self.assertTrue('vector< int >' == v_traits.remove_defaults(v_int))
        v_string = self.global_ns.typedef('v_string')
        self.assertTrue(
            'vector< std::string >' == v_traits.remove_defaults(v_string))
        v_v_int = self.global_ns.typedef('v_v_int')
        self.assertTrue(
            'vector< std::vector< int > >' ==
            v_traits.remove_defaults(v_v_int))

    def test_list(self):
        l_int = self.global_ns.typedef('l_int')
        l_traits = declarations.list_traits
        self.assertTrue('list< int >' == l_traits.remove_defaults(l_int))
        l_wstring = self.global_ns.typedef('l_wstring')
        self.assertTrue(
            'list< std::wstring >' == l_traits.remove_defaults(l_wstring))

    def test_deque(self):
        d_v_int = self.global_ns.typedef('d_v_int')
        d_v_traits = declarations.deque_traits
        self.assertTrue(
            'deque< std::vector< int > >' ==
            d_v_traits.remove_defaults(d_v_int))
        d_l_string = self.global_ns.typedef('d_l_string')
        self.assertTrue(
            'deque< std::list< std::string > >' ==
            d_v_traits.remove_defaults(d_l_string))

    def test_queue(self):
        q_int = self.global_ns.typedef('q_int')
        q_traits = declarations.queue_traits
        self.assertTrue('queue< int >' == q_traits.remove_defaults(q_int))
        q_string = self.global_ns.typedef('q_string')
        self.assertTrue(
            'queue< std::string >' == q_traits.remove_defaults(q_string))

    def test_priority_queue(self):
        pq_int = self.global_ns.typedef('pq_int')
        pq_traits = declarations.priority_queue_traits
        self.assertTrue(
            'priority_queue< int >' == pq_traits.remove_defaults(pq_int))
        pq_string = self.global_ns.typedef('pq_string')
        self.assertTrue(
            'priority_queue< std::string >' ==
            pq_traits.remove_defaults(pq_string))

    def test_set(self):
        s_v_int = self.global_ns.typedef('s_v_int')
        self.assertTrue(
            'set< std::vector< int > >' ==
            declarations.set_traits.remove_defaults(s_v_int))
        s_string = self.global_ns.typedef('s_string')
        self.assertTrue(
            'set< std::string >' ==
            declarations.set_traits.remove_defaults(s_string))

    def test_multiset(self):
        ms_v_int = self.global_ns.typedef('ms_v_int')
        ms_v_traits = declarations.multiset_traits
        self.assertTrue(
            'multiset< std::vector< int > >' ==
            ms_v_traits.remove_defaults(ms_v_int))
        ms_string = self.global_ns.typedef('ms_string')
        self.assertTrue(
            'multiset< std::string >' ==
            ms_v_traits.remove_defaults(ms_string))

    def test_map(self):
        m_i2d = self.global_ns.typedef('m_i2d')
        self.assertTrue(
            'map< int, double >' ==
            declarations.map_traits.remove_defaults(m_i2d))
        m_wstr2d = self.global_ns.typedef('m_wstr2d')
        self.assertTrue(
            'map< std::wstring, double >' ==
            declarations.map_traits.remove_defaults(m_wstr2d))
        m_v_i2m_wstr2d = self.global_ns.typedef('m_v_i2m_wstr2d')
        m = 'map< const std::vector< int >, std::map< std::wstring, double > >'
        self.assertTrue(
            m == declarations.map_traits.remove_defaults(m_v_i2m_wstr2d))

    def test_multimap(self):
        mm_i2d = self.global_ns.typedef('mm_i2d')
        mm_traits = declarations.multimap_traits
        self.assertTrue(
            'multimap< int, double >' == mm_traits.remove_defaults(mm_i2d))
        mm_wstr2d = self.global_ns.typedef('mm_wstr2d')
        self.assertTrue(
            'multimap< const std::wstring, double >' ==
            mm_traits.remove_defaults(mm_wstr2d))
        mm_v_i2mm_wstr2d = self.global_ns.typedef('mm_v_i2mm_wstr2d')
        self.assertTrue(
            ('multimap< const std::vector< int >, ' +
                'const std::multimap< const std::wstring, double > >') ==
            mm_traits.remove_defaults(mm_v_i2mm_wstr2d))

    def test_hash_set(self):
        hs_v_int = self.global_ns.typedef('hs_v_int')
        if 'CastXML' in utils.xml_generator:
            hs_traits = declarations.unordered_set_traits
            name = 'unordered_set'
        else:
            hs_traits = declarations.hash_set_traits
            name = 'hash_set'
        self.assertTrue(
            (name + '< std::vector< int > >') ==
            hs_traits.remove_defaults(hs_v_int),
            hs_traits.remove_defaults(hs_v_int))
        hs_string = self.global_ns.typedef('hs_string')
        self.assertTrue(
            (name + '< std::string >') == hs_traits.remove_defaults(hs_string))

    def test_hash_multiset(self):
        mhs_v_int = self.global_ns.typedef('mhs_v_int')
        if 'CastXML' in utils.xml_generator:
            mhs_traits = declarations.unordered_multiset_traits
            name = 'unordered_multiset'
        else:
            mhs_traits = declarations.hash_multiset_traits
            name = 'hash_multiset'
        self.assertTrue(
            (name + '< std::vector< int > >') ==
            mhs_traits.remove_defaults(mhs_v_int))
        mhs_string = self.global_ns.typedef('mhs_string')
        self.assertTrue(
            (name + '< std::string >') ==
            mhs_traits.remove_defaults(mhs_string))

    def test_hash_map(self):
        hm_i2d = self.global_ns.typedef('hm_i2d')
        if 'CastXML' in utils.xml_generator:
            hm_traits = declarations.unordered_map_traits
            name = 'unordered_map'
        else:
            hm_traits = declarations.hash_map_traits
            name = 'hash_map'
        self.assertTrue(
            (name + '< int, double >') == hm_traits.remove_defaults(hm_i2d))
        hm_wstr2d = self.global_ns.typedef('hm_wstr2d')
        self.assertTrue(
            (name + '< std::wstring, double >') ==
            hm_traits.remove_defaults(hm_wstr2d))

    def test_hash_multimap(self):
        hmm_i2d = self.global_ns.typedef('hmm_i2d')
        if 'CastXML' in utils.xml_generator:
            hmm_traits = declarations.unordered_multimap_traits
            name = 'unordered_multimap'
        else:
            hmm_traits = declarations.hash_multimap_traits
            name = 'hash_multimap'
        self.assertTrue(
            (name + '< int, double >') ==
            hmm_traits.remove_defaults(hmm_i2d))
        hmm_wstr2d = self.global_ns.typedef('hmm_wstr2d')
        self.assertTrue(
            (name + '< const std::wstring, double >') ==
            hmm_traits.remove_defaults(hmm_wstr2d))

        hmm_v_i2mm_wstr2d = self.global_ns.typedef('hmm_v_i2mm_wstr2d')

        hmm_traits_value = hmm_traits.remove_defaults(hmm_v_i2mm_wstr2d)

        possible_values = (
            name + '< const std::vector< int >, ' +
            'const __gnu_cxx::' + name + '< const std::wstring, double > >',
            name + '< const std::vector< int >, ' +
            'const std::' + utils.get_tr1(hmm_traits_value) + name +
            '< const std::wstring, double > >',
            name + '< const std::vector< int >, ' +
            'const stdext::' + name + '< const std::wstring, double > >')

        self.assertTrue(hmm_traits_value in possible_values, hmm_traits_value)


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case

from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = 'non_copyable_classes.hpp'
        self.global_ns = None

    def setUp(self):
        if not self.global_ns:
            decls = parser.parse([self.header], self.config)
            self.global_ns = declarations.get_global_namespace(decls)
            self.global_ns.init_optimizer()

    def test(self):

        """
        Search for classes which can not be copied.

        See bug #13

        1) non copyable class
        2) non copyable const variable (fundamental type)
        3) non copyable const variable (class type)
        4) non copyable const variable (array type)
        5) non copyable const variable (class type)

        """

        main_foo_1 = self.global_ns.class_('MainFoo1')
        self.assertTrue(declarations.is_noncopyable(main_foo_1))

        main_foo_2 = self.global_ns.class_('MainFoo2')
        self.assertTrue(declarations.is_noncopyable(main_foo_2))

        main_foo_3 = self.global_ns.class_('MainFoo3')
        self.assertTrue(declarations.is_noncopyable(main_foo_3))

        main_foo_4 = self.global_ns.class_('MainFoo4')
        self.assertTrue(declarations.is_noncopyable(main_foo_4))

        main_foo_5 = self.global_ns.class_('MainFoo5')
        self.assertTrue(declarations.is_noncopyable(main_foo_5))


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import os
import fnmatch
import unittest
import subprocess


class Test(unittest.TestCase):

    def test_example(self):
        """Runs the example in the docs directory"""

        env = os.environ.copy()

        # Get the path to current directory
        path = os.path.dirname(os.path.realpath(__file__))
        # Set the COVERAGE_PROCESS_START env. variable.
        # Allows to cover files run in a subprocess
        # http://nedbatchelder.com/code/coverage/subprocess.html
        env["COVERAGE_PROCESS_START"] = path + "/../.coveragerc"

        # Find all the examples files
        file_paths = []
        for root, dirnames, filenames in os.walk(path + "/../docs/examples"):
            for file_path in fnmatch.filter(filenames, '*.py'):
                file_paths.append(os.path.join(root, file_path))

        for file_path in file_paths:
            return_code = subprocess.call(
                ["python", path + "/example_tester_wrap.py", file_path],
                env=env)
            self.assertFalse(
                return_code,
                msg="The example %s did not run correctly" % file_path)


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import os
import unittest
import parser_test_case
from pygccxml import parser
from pygccxml import declarations


class tester_1_t(parser_test_case.parser_test_case_t):
    COMPILATION_MODE = parser.COMPILATION_MODE.ALL_AT_ONCE
    declarations = None

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = 'bit_fields.hpp'
        self.declarations = None

    def setUp(self):
        if not self.declarations:
            self.declarations = parser.parse([self.header], self.config)

    def test(self):
        criteria = declarations.variable_matcher_t(
            name='x',
            decl_type='unsigned int')
        x = declarations.matcher.get_single(criteria, self.declarations)

        comp_str = (
            '(decl type==variable_t) and (name==x) and ' +
            '(value type==unsigned int)')
        self.assertTrue(str(criteria) == comp_str)

        criteria = declarations.variable_matcher_t(
            name='::bit_fields::fields_t::x',
            decl_type=declarations.unsigned_int_t(),
            header_dir=os.path.dirname(
                x.location.file_name),
            header_file=x.location.file_name)

        x = declarations.matcher.get_single(criteria, self.declarations)
        self.assertTrue(x, "Variable was not found.")

        self.assertTrue('public' == x.access_type)


class tester_2_t(parser_test_case.parser_test_case_t):
    COMPILATION_MODE = parser.COMPILATION_MODE.ALL_AT_ONCE
    global_ns = None

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = 'vector_traits.hpp'
        self.global_ns = None

    def setUp(self):
        if not self.global_ns:
            self.global_ns = declarations.get_global_namespace(
                parser.parse([self.header], self.config))

    def test_no_defaults(self):
        self.global_ns.decls(lambda decl: 'vector<' in decl.name)
        self.global_ns.decl('vector< _0_ >')
        self.global_ns.class_('vector< std::vector< int > >')
        self.global_ns.class_('vector< std::string >')
        self.global_ns.decl('vector< const int >')


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(tester_1_t))
    suite.addTest(unittest.makeSuite(tester_2_t))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case

from pygccxml import parser
from pygccxml import declarations


class source_reader_tester_t(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = 'unnamed_enums_bug1.hpp'
        self.global_ns = None

    def setUp(self):
        if not self.global_ns:
            reader = parser.source_reader_t(self.config)
            decls = reader.read_file(self.header)
            self.global_ns = declarations.get_global_namespace(decls)
            self.global_ns.init_optimizer()

    def test(self):
        names = []
        enums = self.global_ns.enums()
        for enum in enums:
            names.extend(list(enum.get_name2value_dict().keys()))
        self.assertTrue(len(names) == 4)
        self.assertTrue('x1' in names)
        self.assertTrue('x2' in names)
        self.assertTrue('y1' in names)
        self.assertTrue('y2' in names)


class project_reader_1_tester_t(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = 'unnamed_enums_bug1.hpp'
        self.global_ns = None

    def setUp(self):
        if not self.global_ns:
            decls = parser.parse([self.header], self.config)
            self.global_ns = declarations.get_global_namespace(decls)
            self.global_ns.init_optimizer()

    def test(self):
        names = []
        for enum in self.global_ns.enums():
            names.extend(list(enum.get_name2value_dict().keys()))
        self.assertTrue(len(names) == 4)
        self.assertTrue('x1' in names)
        self.assertTrue('x2' in names)
        self.assertTrue('y1' in names)
        self.assertTrue('y2' in names)


class project_reader_3_tester_t(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.headers = [
            'unnamed_enums_bug1.hpp',
            'unnamed_enums_bug2.hpp',
            'unnamed_enums_bug1.hpp']
        self.global_ns = None

    def setUp(self):
        if not self.global_ns:
            decls = parser.parse(self.headers, self.config)
            self.global_ns = declarations.get_global_namespace(decls)
            self.global_ns.init_optimizer()

    def test(self):
        names = []
        enums = self.global_ns.enums()
        list(map(
            lambda enum: names.extend(list(enum.get_name2value_dict().keys())),
            enums))
        self.assertTrue(len(names) == 6)
        self.assertTrue('x1' in names)
        self.assertTrue('x2' in names)
        self.assertTrue('y1' in names)
        self.assertTrue('y2' in names)
        self.assertTrue('z1' in names)
        self.assertTrue('z2' in names)


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(source_reader_tester_t))
    suite.addTest(unittest.makeSuite(project_reader_1_tester_t))
    suite.addTest(unittest.makeSuite(project_reader_3_tester_t))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case

from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = "declaration_string.hpp"
        self.global_ns = None

    def setUp(self):
        if not self.global_ns:
            decls = parser.parse([self.header], self.config)
            self.global_ns = declarations.get_global_namespace(decls)
            self.global_ns.init_optimizer()

    def test(self):
        """
        Test the create_decl_string method.

        """

        myfunc = self.global_ns.free_function("myfunc")

        decl = declarations.free_function_type_t.create_decl_string(
            myfunc.return_type, myfunc.argument_types)

        self.assertTrue(decl != "('int (*)( int,int )', 'int (*)( int,int )')")

        box = self.global_ns.class_("Box")
        myinternfunc = box.member_function("myinternfunc")
        decl = declarations.member_function_type_t.create_decl_string(
            myinternfunc.return_type,
            box.decl_string,
            myinternfunc.argument_types,
            myinternfunc.has_const)

        self.assertTrue(decl != "short int ( ::Box::* )(  ) ")


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case
from pygccxml import parser
from pygccxml import declarations
from pygccxml import utils


class Test(parser_test_case.parser_test_case_t):
    global_ns = None

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.headers = ['remove_template_defaults.hpp', 'indexing_suites2.hpp']

    def setUp(self):
        if not Test.global_ns:
            decls = parser.parse(self.headers, self.config)
            Test.global_ns = declarations.get_global_namespace(decls)
            Test.global_ns.init_optimizer()

    def __cmp_traits(self, typedef, expected, partial_name, key_type=None):
        if utils.is_str(typedef):
            typedef = self.global_ns.typedef(typedef)
        traits = declarations.find_container_traits(typedef)
        self.assertTrue(
            traits,
            'container traits for "%s" not found' %
            str(typedef))
        self.assertTrue(
            traits is expected,
            'container "%s", expected %s_traits, got %s_traits' %
            (str(typedef),
             expected.name(),
             traits.name()))
        cls = declarations.remove_declarated(typedef)
        self.assertTrue(declarations.find_container_traits(cls) is expected)
        self.assertTrue(cls.partial_name == partial_name)
        cls = traits.class_declaration(cls)

        self.assertTrue(traits.element_type(typedef))
        self.assertTrue(
            cls.cache.container_element_type,
            "For some reason cache was not updated")

        if key_type:
            self.assertTrue(traits.is_mapping(typedef))
            real_key_type = traits.key_type(typedef)
            self.assertTrue(
                real_key_type.decl_string == key_type,
                'Error extracting key type.  Expected type "%s", got "%s"' %
                (key_type,
                 real_key_type.decl_string))
            self.assertTrue(
                cls.cache.container_key_type,
                "For some reason cache was not updated")
        else:
            self.assertTrue(traits.is_sequence(typedef))

    def test_find_traits(self):
        self.__cmp_traits('v_int', declarations.vector_traits, "vector< int >")
        self.__cmp_traits('l_int', declarations.list_traits, "list< int >")
        self.__cmp_traits(
            'd_v_int',
            declarations.deque_traits,
            "deque< std::vector< int > >")
        self.__cmp_traits('q_int', declarations.queue_traits, "queue< int >")
        self.__cmp_traits(
            'pq_int',
            declarations.priority_queue_traits,
            "priority_queue< int >")
        self.__cmp_traits(
            's_v_int',
            declarations.set_traits,
            "set< std::vector< int > >")
        self.__cmp_traits(
            'ms_v_int',
            declarations.multiset_traits,
            "multiset< std::vector< int > >")
        self.__cmp_traits(
            'm_i2d',
            declarations.map_traits,
            "map< int, double >",
            'int')
        self.__cmp_traits(
            'mm_i2d',
            declarations.multimap_traits,
            "multimap< int, double >",
            'int')

        if "CastXML" in utils.xml_generator:
            self.__cmp_traits(
                'hs_v_int',
                declarations.unordered_set_traits,
                "unordered_set< std::vector< int > >")
        else:
            self.__cmp_traits(
                'hs_v_int',
                declarations.hash_set_traits,
                "hash_set< std::vector< int > >")

        if "CastXML" in utils.xml_generator:
            self.__cmp_traits(
                'mhs_v_int',
                declarations.unordered_multiset_traits,
                "unordered_multiset< std::vector< int > >")
        else:
            self.__cmp_traits(
                'mhs_v_int',
                declarations.hash_multiset_traits,
                "hash_multiset< std::vector< int > >")

        if "CastXML" in utils.xml_generator:
            self.__cmp_traits(
                'hm_i2d',
                declarations.unordered_map_traits,
                "unordered_map< int, double >",
                'int')
        else:
            self.__cmp_traits(
                'hm_i2d',
                declarations.hash_map_traits,
                "hash_map< int, double >",
                'int')

        if "CastXML" in utils.xml_generator:
            self.__cmp_traits(
                'hmm_i2d',
                declarations.unordered_multimap_traits,
                "unordered_multimap< int, double >",
                'int')
        else:
            self.__cmp_traits(
                'hmm_i2d',
                declarations.hash_multimap_traits,
                "hash_multimap< int, double >",
                'int')

    def test_multimap(self):
        m = self.global_ns.class_(
            lambda decl: decl.name.startswith('multimap'))
        declarations.find_container_traits(m)
        self.assertTrue(m.partial_name == 'multimap< int, int >')

    def test_recursive_partial_name(self):
        f1 = self.global_ns.free_fun('f1')
        t1 = declarations.class_traits.get_declaration(
            f1.arguments[0].decl_type)
        self.assertTrue(
            'type< std::set< std::vector< int > > >' == t1.partial_name)

    def test_remove_defaults_partial_name_namespace(self):
        f2 = self.global_ns.free_fun('f2')
        type_info = f2.return_type
        traits = declarations.find_container_traits(type_info)
        cls = traits.class_declaration(type_info)
        # traits.remove_defaults(type_info)
        decl_string = cls.partial_decl_string
        key_type_string = traits.key_type(type_info).partial_decl_string
        self.assertTrue(
            decl_string.startswith('::std::'),
            "declaration string %r doesn't start with 'std::'" %
            decl_string)
        self.assertTrue(
            key_type_string.startswith('::std::'),
            "key type string %r doesn't start with 'std::'" %
            key_type_string)

    @staticmethod
    def test_from_ogre():
        x = (
            'map<std::string, bool (*)(std::string&, ' +
            'Ogre::MaterialScriptContext&), std::less<std::string>, ' +
            'std::allocator<std::pair<std::string const, bool (*)' +
            '(std::string&, Ogre::MaterialScriptContext&)> > >')
        ct = declarations.find_container_traits(x)
        ct.remove_defaults(x)

    def test_infinite_loop(self):
        rt = self.global_ns.free_fun('test_infinite_loop').return_type
        map_traits = declarations.find_container_traits(rt)
        self.assertTrue(map_traits is declarations.map_traits)
        elem = map_traits.element_type(rt)
        self.assertTrue(elem.decl_string == 'int')


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

"""
defines classes, that describes C++ classes

This modules contains definition for next C++ declarations:
    - class definition
    - class declaration
    - small helper class for describing C++ class hierarchy
"""

import warnings
from . import scopedef
from . import declaration_utils
from . import declaration
from . import templates
from . import cpptypes
from .. import utils


class ACCESS_TYPES(object):

    """class that defines "access" constants"""
    PUBLIC = "public"
    PRIVATE = "private"
    PROTECTED = "protected"
    ALL = [PUBLIC, PRIVATE, PROTECTED]


class CLASS_TYPES(object):

    """class that defines "class" type constants"""
    CLASS = "class"
    STRUCT = "struct"
    UNION = "union"
    ALL = [CLASS, STRUCT, UNION]


def get_partial_name(name):
    from . import container_traits  # prevent cyclic dependencies
    ct = container_traits.find_container_traits(name)
    if ct:
        return ct.remove_defaults(name)
    elif templates.is_instantiation(name):
        tmpl_name, args = templates.split(name)
        for i, arg_name in enumerate(args):
            args[i] = get_partial_name(arg_name.strip())
        return templates.join(tmpl_name, args)
    else:
        return name


class hierarchy_info_t(object):

    """describes class relationship"""

    def __init__(self, related_class=None, access=None, is_virtual=False):
        """creates class that contains partial information about class
        relationship"""
        if related_class:
            assert(isinstance(related_class, class_t))
        self._related_class = related_class
        if access:
            assert(access in ACCESS_TYPES.ALL)
        self._access = access
        self._is_virtual = is_virtual
        self._declaration_path = None
        self._declaration_path_hash = None

    def __eq__(self, other):
        if not isinstance(other, hierarchy_info_t):
            return False
        return (self.declaration_path_hash ==
                other.declaration_path_hash) \
            and self._declaration_path == other._declaration_path \
            and self._access == other._access \
            and self._is_virtual == other._is_virtual

    def __hash__(self):
        return self.declaration_path_hash

    def __ne__(self, other):
        return not self.__eq__(other)

    def __lt__(self, other):
        if not isinstance(other, self.__class__):
            return self.__class__.__name__ < other.__class__.__name__
        return (self.declaration_path, self.access, self.is_virtual) < \
            (other.declaration_path, other.access, other.is_virtual)

    @property
    def related_class(self):
        """reference to base or derived :class:`class <class_t>`"""
        return self._related_class

    @related_class.setter
    def related_class(self, new_related_class):
        if new_related_class:
            assert(isinstance(new_related_class, class_t))
        self._related_class = new_related_class
        self._declaration_path = None
        self._declaration_path_hash = None

    @property
    def access(self):
        return self._access

    @access.setter
    def access(self, new_access):
        assert(new_access in ACCESS_TYPES.ALL)
        self._access = new_access

    # TODO: Why is there an access_type / access which are the same ?
    @property
    def access_type(self):
        """describes :class:`hierarchy type <ACCESS_TYPES>`"""
        return self.access

    @access_type.setter
    def access_type(self, new_access_type):
        self.access = new_access_type

    # TODO: check whether GCC XML support this and if so parser this
    # information
    @property
    def is_virtual(self):
        """indicates whether the inheritance is virtual or not"""
        return self._is_virtual

    @is_virtual.setter
    def is_virtual(self, new_is_virtual):
        self._is_virtual = new_is_virtual

    @property
    def declaration_path(self):
        if self._declaration_path is None:
            self._declaration_path = declaration_utils.declaration_path(
                self.related_class)
        return self._declaration_path

    @property
    def declaration_path_hash(self):
        if self._declaration_path_hash is None:
            self._declaration_path_hash = hash(tuple(self.declaration_path))
        return self._declaration_path_hash


class class_declaration_t(declaration.declaration_t):

    """describes class declaration"""

    def __init__(self, name=''):
        """creates class that describes C++ class declaration
        ( and not definition )"""
        declaration.declaration_t.__init__(self, name)
        self._aliases = []
        self._container_traits = None  # Deprecated
        self._container_traits_set = False  # Deprecated
        self._container_traits_cache = None

    def _get__cmp__items(self):
        """implementation details"""
        return []

    def i_depend_on_them(self, recursive=True):
        return []

    @property
    def aliases(self):
        """List of :class:`aliases <typedef_t>` to this instance"""
        return self._aliases

    @aliases.setter
    def aliases(self, new_aliases):
        self._aliases = new_aliases

    @property
    def container_traits(self):
        """reference to :class:`container_traits_impl_t` or None"""

        # Deprecated since 1.8.0. Will be removed in 1.9.0
        warnings.warn(
            "The container_traits attribute is deprecated. \n" +
            "Please use the find_container_traits function from the"
            "declarations module instead.",
            DeprecationWarning)

        if self._container_traits_set is False:
            from . import container_traits  # prevent cyclic dependencies
            self._container_traits_set = True
            self._container_traits = container_traits.find_container_traits(
                self)
            self._container_traits_cache = self._container_traits
        return self._container_traits

    def _get_partial_name_impl(self):
        return get_partial_name(self.name)


class class_t(scopedef.scopedef_t):

    """describes class definition"""

    # Can be set from outside
    USE_DEMANGLED_AS_NAME = True

    def __init__(
            self,
            name='',
            class_type=CLASS_TYPES.CLASS,
            is_abstract=False):
        """creates class that describes C++ class definition"""
        scopedef.scopedef_t.__init__(self, name)
        if class_type:
            assert(class_type in CLASS_TYPES.ALL)
        self._class_type = class_type
        self._bases = []
        self._derived = []
        self._is_abstract = is_abstract
        self._public_members = []
        self._private_members = []
        self._protected_members = []
        self._aliases = []
        self._byte_size = 0
        self._byte_align = 0
        self._container_traits_cache = None
        self._container_traits = None  # Deprecated
        self._container_traits_set = False  # Deprecated
        self._recursive_bases = None
        self._recursive_derived = None
        self._use_demangled_as_name = False

    @property
    def use_demangled_as_name(self):
        if "GCC" in utils.xml_generator:
            return class_t.USE_DEMANGLED_AS_NAME
        elif "CastXML" in utils.xml_generator:
            return False

    @use_demangled_as_name.setter
    def use_demangled_as_name(self, use_demangled_as_name):
        self._use_demangled_as_name = use_demangled_as_name

    def _get_name_impl(self):
        if not self._name:  # class with empty name
            return self._name
        elif self.use_demangled_as_name and self.demangled:

            if not self.cache.demangled_name:
                fname = declaration_utils.full_name(self.parent)
                if fname.startswith('::') and \
                        not self.demangled.startswith('::'):
                    fname = fname[2:]
                if self.demangled.startswith(fname):
                    tmp = self.demangled[len(fname):]  # demangled::name
                    if tmp.startswith('::'):
                        tmp = tmp[2:]
                    if '<' not in tmp and '<' in self._name:
                        # we have template class, but for some reason demangled
                        # name doesn't contain any template
                        # This happens for std::string class, but this breaks
                        # other cases, because this behaviour is not consistent
                        self.cache.demangled_name = self._name
                        return self.cache.demangled_name
                    else:
                        self.cache.demangled_name = tmp
                        return tmp
                else:
                    self.cache.demangled_name = self._name
                    return self._name
            else:
                return self.cache.demangled_name
        else:
            return self._name

    def __str__(self):
        name = declaration_utils.full_name(self)
        if name[:2] == "::":
            name = name[2:]
        return "%s [%s]" % (name, self.class_type)

    def _get__cmp__scope_items(self):
        """implementation details"""
        return [
                self.class_type,
                [declaration_utils.declaration_path(base.related_class) for
                 base in self.bases].sort(),
                [declaration_utils.declaration_path(derive.related_class) for
                 derive in self.derived].sort(),
                self.is_abstract,
                self.public_members.sort(),
                self.private_members.sort(),
                self.protected_members.sort()]

    def __eq__(self, other):
        if not scopedef.scopedef_t.__eq__(self, other):
            return False
        return self.class_type == other.class_type \
            and [declaration_utils.declaration_path(base.related_class) for
                 base in self.bases].sort() \
            == [declaration_utils.declaration_path(base.related_class) for
                base in other.bases].sort() \
            and [declaration_utils.declaration_path(derive.related_class) for
                 derive in self.derived].sort() \
            == [declaration_utils.declaration_path(derive.related_class) for
                derive in other.derived].sort() \
            and self.is_abstract == other.is_abstract \
            and self.public_members.sort() \
            == other.public_members.sort() \
            and self.private_members.sort() \
            == other.private_members.sort() \
            and self.protected_members.sort() \
            == other.protected_members.sort()

    def __hash__(self):
        return hash(self.class_type)

    @property
    def class_type(self):
        """describes class :class:`type <CLASS_TYPES>`"""
        return self._class_type

    @class_type.setter
    def class_type(self, new_class_type):
        if new_class_type:
            assert(new_class_type in CLASS_TYPES.ALL)
        self._class_type = new_class_type

    @property
    def bases(self):
        """list of :class:`base classes <hierarchy_info_t>`"""
        return self._bases

    @bases.setter
    def bases(self, new_bases):
        self._bases = new_bases

    @property
    def recursive_bases(self):
        """list of all :class:`base classes <hierarchy_info_t>`"""
        if self._recursive_bases is None:
            to_go = self.bases[:]
            all_bases = []
            while to_go:
                base = to_go.pop()
                if base not in all_bases:
                    all_bases.append(base)
                    to_go.extend(base.related_class.bases)
            self._recursive_bases = all_bases
        return self._recursive_bases

    @property
    def derived(self):
        """list of :class:`derived classes <hierarchy_info_t>`"""
        return self._derived

    @derived.setter
    def derived(self, new_derived):
        self._derived = new_derived

    @property
    def recursive_derived(self):
        """list of all :class:`derive classes <hierarchy_info_t>`"""
        if self._recursive_derived is None:
            to_go = self.derived[:]
            all_derived = []
            while to_go:
                derive = to_go.pop()
                if derive not in all_derived:
                    all_derived.append(derive)
                    to_go.extend(derive.related_class.derived)
            self._recursive_derived = all_derived
        return self._recursive_derived

    @property
    def is_abstract(self):
        """describes whether class abstract or not"""
        return self._is_abstract

    @is_abstract.setter
    def is_abstract(self, is_abstract):
        self._is_abstract = is_abstract

    @property
    def public_members(self):
        """list of all public :class:`members <declarationt_>`"""
        return self._public_members

    @public_members.setter
    def public_members(self, new_public_members):
        self._public_members = new_public_members

    @property
    def private_members(self):
        """list of all private :class:`members <declarationt_>`"""
        return self._private_members

    @private_members.setter
    def private_members(self, new_private_members):
        self._private_members = new_private_members

    @property
    def protected_members(self):
        """list of all protected :class:`members <declarationt_>`"""
        return self._protected_members

    @protected_members.setter
    def protected_members(self, new_protected_members):
        self._protected_members = new_protected_members

    @property
    def aliases(self):
        """List of :class:`aliases <typedef_t>` to this instance"""
        return self._aliases

    @aliases.setter
    def aliases(self, new_aliases):
        self._aliases = new_aliases

    @property
    def byte_size(self):
        """Size of this class in bytes @type: int"""
        return self._byte_size

    @byte_size.setter
    def byte_size(self, new_byte_size):
        self._byte_size = new_byte_size

    @property
    def byte_align(self):
        """Alignment of this class in bytes @type: int"""
        return self._byte_align

    @byte_align.setter
    def byte_align(self, new_byte_align):
        self._byte_align = new_byte_align

    def _get_declarations_impl(self):
        return self.get_members()

    def get_members(self, access=None):
        """
        returns list of members according to access type

        If access equals to None, then returned list will contain all members.
        You should not modify the list content, otherwise different
        optimization data will stop work and may to give you wrong results.

        :param access: describes desired members
        :type access: :class:ACCESS_TYPES

        :rtype: [ members ]
        """
        if access == ACCESS_TYPES.PUBLIC:
            return self.public_members
        elif access == ACCESS_TYPES.PROTECTED:
            return self.protected_members
        elif access == ACCESS_TYPES.PRIVATE:
            return self.private_members
        else:
            all_members = []
            all_members.extend(self.public_members)
            all_members.extend(self.protected_members)
            all_members.extend(self.private_members)
            return all_members

    def adopt_declaration(self, decl, access):
        """adds new declaration to the class

        :param decl: reference to a :class:`declaration_t`

        :param access: member access type
        :type access: :class:ACCESS_TYPES
        """
        if access == ACCESS_TYPES.PUBLIC:
            self.public_members.append(decl)
        elif access == ACCESS_TYPES.PROTECTED:
            self.protected_members.append(decl)
        elif access == ACCESS_TYPES.PRIVATE:
            self.private_members.append(decl)
        else:
            raise RuntimeError("Invalid access type: %s." % access)
        decl.parent = self
        decl.cache.reset()
        decl.cache.access_type = access

    def remove_declaration(self, decl):
        """
        removes decl from  members list

        :param decl: declaration to be removed
        :type decl: :class:`declaration_t`
        """

        access_type = self.find_out_member_access_type(decl)
        if access_type == ACCESS_TYPES.PUBLIC:
            container = self.public_members
        elif access_type == ACCESS_TYPES.PROTECTED:
            container = self.protected_members
        else:  # decl.cache.access_type == ACCESS_TYPES.PRVATE
            container = self.private_members
        del container[container.index(decl)]
        decl.cache.reset()

    def find_out_member_access_type(self, member):
        """
        returns member access type

        :param member: member of the class
        :type member: :class:`declaration_t`

        :rtype: :class:ACCESS_TYPES
        """
        assert member.parent is self
        if not member.cache.access_type:
            if member in self.public_members:
                access_type = ACCESS_TYPES.PUBLIC
            elif member in self.protected_members:
                access_type = ACCESS_TYPES.PROTECTED
            elif member in self.private_members:
                access_type = ACCESS_TYPES.PRIVATE
            else:
                raise RuntimeError(
                    "Unable to find member within internal members list.")
            member.cache.access_type = access_type
            return access_type
        else:
            return member.cache.access_type

    def __find_out_member_dependencies(self, access_type):
        members = self.get_members(access_type)
        answer = []
        for mem in members:
            answer.extend(mem.i_depend_on_them(recursive=True))
        member_ids = set([id(m) for m in members])
        for dependency in answer:
            if id(dependency.declaration) in member_ids:
                dependency.access_type = access_type
        return answer

    def i_depend_on_them(self, recursive=True):

        answer = []

        for base in self.bases:
            answer.append(
                dependency_info_t(
                    self,
                    base.related_class,
                    base.access_type,
                    "base class"))

        if recursive:
            for access_type in ACCESS_TYPES.ALL:
                answer.extend(self.__find_out_member_dependencies(access_type))

        return answer

    @property
    def container_traits(self):
        """reference to :class:`container_traits_impl_t` or None"""

        # Deprecated since 1.8.0. Will be removed in 1.9.0
        warnings.warn(
            "The container_traits attribute is deprecated. \n" +
            "Please use the find_container_traits function from the"
            "declarations module instead.",
            DeprecationWarning)

        if self._container_traits_set is False:
            from . import container_traits  # prevent cyclic dependencies
            self._container_traits_set = True
            self._container_traits = container_traits.find_container_traits(
                self)
            self._container_traits_cache = self.container_traits
        return self._container_traits

    def find_copy_constructor(self):

        # Deprecated since 1.8.0. Will be removed in 1.9.0
        warnings.warn(
            "The find_copy_constructor method is deprecated. \n" +
            "Please use the find_copy_constructor function from the"
            "declarations module instead.",
            DeprecationWarning)

        from . import type_traits_classes  # prevent cyclic dependencies
        return type_traits_classes.find_copy_constructor(self)

    def find_trivial_constructor(self):

        # Deprecated since 1.8.0. Will be removed in 1.9.0
        warnings.warn(
            "The find_trivial_constructor method is deprecated. \n" +
            "Please use the find_trivial_constructor function from the"
            "declarations module instead.",
            DeprecationWarning)

        from . import type_traits_classes  # prevent cyclic dependencies
        return type_traits_classes.find_trivial_constructor(self)

    def _get_partial_name_impl(self):
        from . import type_traits  # prevent cyclic dependencies
        if type_traits.is_std_string(self):
            return 'string'
        elif type_traits.is_std_wstring(self):
            return 'wstring'
        else:
            return get_partial_name(self.name)

    def find_noncopyable_vars(self):
        """returns list of all `noncopyable` variables"""

        # Deprecated since 1.8.0. Will be removed in 1.9.0
        warnings.warn(
            "The find_noncopyable_vars method is deprecated. \n" +
            "Please use the find_noncopyable_vars function from the"
            "declarations module instead.",
            DeprecationWarning)

        from . import type_traits_classes  # prevent cyclic dependencies
        type_traits_classes.find_noncopyable_vars(self)

    @property
    def has_vtable(self):
        """True, if class has virtual table, False otherwise"""

        # Deprecated since 1.8.0. Will be removed in 1.9.0
        warnings.warn(
            "The has_vtable argument is deprecated. \n" +
            "Please use the has_vtable function from the declarations \n" +
            "module instead.",
            DeprecationWarning)

        # prevent cyclic import
        from . import type_traits_classes
        return type_traits_classes.has_vtable(self)

    @property
    def top_class(self):
        """reference to a parent class, which contains this class and defined
        within a namespace

        if this class is defined under a namespace, self will be returned"""
        curr = self
        parent = self.parent
        while isinstance(parent, class_t):
            curr = parent
            parent = parent.parent
        return curr

class_types = (class_t, class_declaration_t)


class impl_details(object):

    @staticmethod
    def dig_declarations(depend_on_it):

        # FIXME: prevent cyclic imports
        from . import type_traits

        if isinstance(depend_on_it, declaration.declaration_t):
            return [depend_on_it]
        base_type = type_traits.base_type(
            type_traits.remove_alias(depend_on_it))
        if isinstance(base_type, cpptypes.declarated_t):
            return [base_type.declaration]
        elif isinstance(base_type, cpptypes.calldef_type_t):
            result = []
            result.extend(impl_details.dig_declarations(base_type.return_type))
            for argtype in base_type.arguments_types:
                result.extend(impl_details.dig_declarations(argtype))
            if isinstance(base_type, cpptypes.member_function_type_t):
                result.extend(
                    impl_details.dig_declarations(
                        base_type.class_inst))
            return result
        return []


class dependency_info_t(object):

    def __init__(self, declaration, depend_on_it, access_type=None, hint=None):
        object.__init__(self)

        assert isinstance(
            depend_on_it,
            (class_t,
             cpptypes.type_t))
        self._declaration = declaration
        self._depend_on_it = depend_on_it
        self._access_type = access_type
        self._hint = hint

    @property
    def declaration(self):
        return self._declaration
    # short name
    decl = declaration

    @property
    def depend_on_it(self):
        return self._depend_on_it

    @property
    def access_type(self):
        return self._access_type

    @access_type.setter
    def access_type(self, access_type):
        self._access_type = access_type

    def __str__(self):
        return 'declaration "%s" depends( %s ) on "%s" ' \
               % (self.declaration, self.access_type, self.depend_on_it)

    @property
    def hint(self):
        """The declaration, that report dependency can put some additional
        inforamtion about dependency. It can be used later"""
        return self._hint

    def find_out_depend_on_it_declarations(self):
        """If declaration depends on other declaration and not on some type
        this function will return reference to it. Otherwise None will be
        returned
        """
        return impl_details.dig_declarations(self.depend_on_it)

    @staticmethod
    def i_depend_on_them(decl):
        """Returns set of declarations. every item in the returned set,
        depends on a declaration from the input"""

        to_be_included = set()
        for dependency_info in decl.i_depend_on_them():
            for ddecl in dependency_info.find_out_depend_on_it_declarations():
                if ddecl:
                    to_be_included.add(ddecl)

        if isinstance(decl.parent, class_t):
            to_be_included.add(decl.parent)
        return to_be_included

    @staticmethod
    def we_depend_on_them(decls):
        """Returns set of declarations. every item in the returned set,
        depends on a declaration from the input"""
        to_be_included = set()
        for decl in decls:
            to_be_included.update(dependency_info_t.i_depend_on_them(decl))
        return to_be_included



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import copy
import unittest
import parser_test_case
from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):
    COMPILATION_MODE = parser.COMPILATION_MODE.ALL_AT_ONCE

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = 'declarations_comparison.hpp'

    def test_comparison_declaration_by_declaration(self):
        parsed = parser.parse([self.header], self.config)
        copied = copy.deepcopy(parsed)
        parsed = declarations.make_flatten(parsed)
        copied = declarations.make_flatten(copied)
        parsed.sort()
        copied.sort()
        failuers = []
        for parsed_decl, copied_decl, index in \
                zip(parsed, copied, list(range(len(copied)))):

            if parsed_decl != copied_decl:
                failuers.append(
                    ("__lt__ and/or __qe__ does not working " +
                        "properly in case of %s, %s, index %d") %
                    (parsed_decl.__class__.__name__,
                        copied_decl.__class__.__name__, index))
        self.assertTrue(not failuers, 'Failures: ' + '\n\t'.join(failuers))

    def test_comparison_from_reverse(self):
        parsed = parser.parse([self.header], self.config)
        copied = copy.deepcopy(parsed)
        parsed.sort()
        copied.reverse()
        copied.sort()
        x = parsed[4:6]
        x.sort()
        y = copied[4:6]
        y.sort()
        self.assertTrue(
            parsed == copied,
            "__lt__ and/or __qe__ does not working properly")

    def test___lt__transitivnost(self):
        ns_std = declarations.namespace_t(name='std')
        ns_global = declarations.namespace_t(name='::')
        ns_internal = declarations.namespace_t(name='ns')
        ns_internal.parent = ns_global
        ns_global.declarations.append(ns_internal)
        left2right = [ns_std, ns_global]
        right2left = [ns_global, ns_std]
        left2right.sort()
        right2left.sort()
        self.assertTrue(left2right == right2left, "bug: find me")

    def test_same_declarations_different_intances(self):
        parsed = parser.parse([self.header], self.config)
        copied = copy.deepcopy(parsed)
        self.assertTrue(
            parsed == copied,
            "__lt__ and/or __qe__ does not working properly")


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case

from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):

    global_ns = None

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = 'covariant_returns.hpp'

    def setUp(self):
        if not Test.global_ns:
            decls = parser.parse([self.header], self.config)
            Test.global_ns = declarations.get_global_namespace(decls)
            Test.global_ns.init_optimizer()

    def test_is_same_function(self):
        d = self.global_ns.class_('better_algorithm_t')
        b = self.global_ns.class_('algorithm_t')

        df = d.mem_fun('f')
        bf = b.mem_fun('f')

        self.assertTrue(
            id(df) != id(bf) and declarations.is_same_function(df, bf))


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import os
import sys
import time
import hotshot
import hotshot.stats

this_module_dir_path = os.path.abspath(
    os.path.dirname(sys.modules[__name__].__file__))

sys.path.insert(1, os.path.join(this_module_dir_path, '../../'))
sys.path.insert(2, os.path.join(this_module_dir_path, '../'))

import autoconfig  # nopep8
from pygccxml import parser  # nopep8
from pygccxml import declarations  # nopep8

dcache_file_name = os.path.join(autoconfig.data_directory, 'pygccxml.cache')
if os.path.exists(dcache_file_name):
    os.remove(dcache_file_name)


def test_on_windows_dot_h():
    he = r"2003\Vc7\PlatformSDK\Include\windows.h"
    windows_header = r"D:\Program Files\Microsoft Visual Studio .NET " + he
    clock_prev = time.clock()
    dcache = parser.file_cache_t(dcache_file_name)
    reader = parser.source_reader_t(
        parser.xml_generator_configuration_t(
            xml_generator_path=autoconfig.generator_path),
        dcache)
    reader.read_file(windows_header)
    dcache.flush()
    clock_now = time.clock()
    print('without cache: %f seconds' % (clock_now - clock_prev))

    clock_prev = time.clock()
    dcache = parser.file_cache_t(dcache_file_name)
    reader = parser.source_reader_t(
        parser.xml_generator_configuration_t(
            xml_generator_path=autoconfig.generator_path),
        dcache)
    reader.read_file(windows_header)
    clock_now = time.clock()
    print('with cache   : %f seconds' % (clock_now - clock_prev))

#########################################################################
# testing include_std.hpp


def test_source_on_include_std_dot_hpp():
    include_std_header = os.path.join(
        autoconfig.data_directory,
        'include_std.hpp')
    clock_prev = time.clock()
    dcache = parser.file_cache_t(dcache_file_name)
    reader = parser.source_reader_t(
        parser.xml_generator_configuration_t(
            xml_generator_path=autoconfig.generator_path),
        dcache)
    reader.read_file(include_std_header)
    dcache.flush()
    clock_now = time.clock()
    print('without cache: %f seconds' % (clock_now - clock_prev))

    clock_prev = time.clock()
    dcache = parser.file_cache_t(dcache_file_name)
    reader = parser.source_reader_t(
        parser.xml_generator_configuration_t(
            xml_generator_path=autoconfig.generator_path),
        dcache)
    reader.read_file(include_std_header)
    clock_now = time.clock()
    print('with cache   : %f seconds' % (clock_now - clock_prev))


#########################################################################
# testing include_std.hpp
def test_project_on_include_std_dot_hpp():
    include_std_header = os.path.join(
        autoconfig.data_directory,
        'include_std.hpp')
    clock_prev = time.clock()
    dcache = parser.file_cache_t(dcache_file_name)
    reader = parser.project_reader_t(
        parser.xml_generator_configuration_t(
            xml_generator_path=autoconfig.generator_path),
        dcache)
    reader.read_files([include_std_header])
    dcache.flush()
    clock_now = time.clock()
    print('without cache: %f seconds' % (clock_now - clock_prev))

    clock_prev = time.clock()
    dcache = parser.file_cache_t(dcache_file_name)
    reader = parser.project_reader_t(
        parser.xml_generator_configuration_t(
            xml_generator_path=autoconfig.generator_path),
        dcache)
    reader.read_files([include_std_header])
    clock_now = time.clock()
    print('with cache   : %f seconds' % (clock_now - clock_prev))


def profile_project():
    include_std_header = os.path.join(
        autoconfig.data_directory,
        'include_std.hpp')
    reader = parser.project_reader_t(
        parser.xml_generator_configuration_t(
            xml_generator_path=autoconfig.generator_path))
    reader.read_files([include_std_header])


def profile_project2():
    he = r"2003\Vc7\PlatformSDK\Include\windows.h"
    include_std_header = r"D:\Program Files\Microsoft Visual Studio .NET " + he
    reader = parser.project_reader_t(
        parser.xml_generator_configuration_t(
            xml_generator_path=autoconfig.generator_path))
    reader.read_files([include_std_header])


def test_on_big_file(file_name, count):
    file_name = os.path.join(autoconfig.data_directory, file_name)
    for i in range(count):
        reader = parser.project_reader_t(
            parser.xml_generator_configuration_t(
                xml_generator_path=autoconfig.generator_path))
        decls = reader.read_files([parser.create_gccxml_fc(file_name)])
        global_ns = declarations.get_global_namespace(decls)
        global_ns.init_optimizer()


if __name__ == "__main__":

    # test_on_windows_dot_h()
    # test_source_on_include_std_dot_hpp()
    # test_project_on_include_std_dot_hpp()
    print('running')
    prof = hotshot.Profile('parser.prof')
    prof.runcall(lambda: test_on_big_file('itkImage.xml', 1))
    stats = hotshot.stats.load("parser.prof")
    stats.sort_stats('time', 'calls')
    stats.print_stats(30)
    print('running - done')
    # print 'loading file'
    # pdata = pstats.Stats('pygccxml.profile')
    # print 'loading file - done'
    # print 'striping dirs'
    # pdata.strip_dirs()
    # print 'striping dirs - done'
    # print 'sorting stats'
    # pdata.sort_stats('time').print_stats(476)
    # print 'sorting stats - done'
    # pdata.print_callers('find_all_declarations')



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case

from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):
    global_ns = None

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = 'type_as_exception_bug.h'

    def setUp(self):
        if not Test.global_ns:
            decls = parser.parse([self.header], self.config)
            Test.global_ns = declarations.get_global_namespace(decls)
            Test.global_ns.init_optimizer()

    def test(self):
        buggy = self.global_ns.mem_fun('buggy')
        expression_error = self.global_ns.class_('ExpressionError')
        self.assertTrue(len(buggy.exceptions) == 1)
        err = buggy.exceptions[0]
        self.assertTrue(declarations.is_reference(err))
        err = declarations.remove_declarated(
            declarations.remove_reference(err))
        self.assertTrue(err is expression_error)


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

from timeit import Timer

stmt = """
import unittest
from test_parser import create_suite
unittest.TextTestRunner(verbosity=2).run( create_suite() )
"""

timer = Timer(stmt)
print(timer.timeit(3))




import os
import pep8
import unittest
import fnmatch


class Test(unittest.TestCase):

    def test_pep8_conformance_unitests(self):
        """Pep8 conformance test (unitests)

        Runs on the unittest directory.
        """

        print("\r\n")

        # Get the path to current directory
        path = os.path.dirname(os.path.realpath(__file__))

        self.run_check(path)

    def test_pep8_conformance_pygccxml(self):
        """Pep8 conformance test (pygccxml)

        Runs on the pygccxml directory.
        """

        print("\r\n")

        # Get the path to current directory
        path = os.path.dirname(os.path.realpath(__file__))
        path += "/../pygccxml/"

        self.run_check(path)

    def test_pep8_conformance_example(self):
        """Pep8 conformance test (examples)

        Runs on the example file in the docs.
        """

        print("\r\n")

        # Get the path to current directory
        path = os.path.dirname(os.path.realpath(__file__))
        path += "/../docs/examples/"

        # Find all the examples files
        file_paths = []
        for root, dirnames, filenames in os.walk(path):
            for file_path in fnmatch.filter(filenames, '*.py'):
                file_paths.append(os.path.join(root, file_path))

        for path in file_paths:
            self.run_check(path)

    def test_pep8_conformance_setup(self):
        """Pep8 conformance test (setup)

        Runs on the setup.py file
        """

        print("\r\n")

        # Get the path to current directory
        path = os.path.dirname(os.path.realpath(__file__))
        path += "/../setup.py"

        self.run_check(path)

    def run_check(self, path):
        """Common method to run the pep8 test."""

        pep8style = pep8.StyleGuide()
        result = pep8style.check_files(paths=[path])

        if result.total_errors != 0:
            self.assertEqual(
                result.total_errors, 0,
                "Found code style errors (and warnings).")


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case
from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):
    COMPILATION_MODE = parser.COMPILATION_MODE.ALL_AT_ONCE
    global_ns = None

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = 'string_traits.hpp'
        self.global_ns = None

    def setUp(self):
        if not Test.global_ns:
            decls = parser.parse([self.header], self.config)
            Test.global_ns = declarations.get_global_namespace(decls)
        self.global_ns = Test.global_ns

    def validate_yes(self, ns, controller):
        for typedef in ns.typedefs():
            self.assertTrue(controller(typedef.decl_type))

    def validate_no(self, ns, controller):
        for typedef in ns.typedefs():
            self.assertTrue(not controller(typedef.decl_type))

    def test_string(self):
        string_traits = self.global_ns.namespace('string_traits')
        self.validate_yes(
            string_traits.namespace('yes'),
            declarations.is_std_string)
        self.validate_no(
            string_traits.namespace('no'),
            declarations.is_std_string)

    def test_wstring(self):
        wstring_traits = self.global_ns.namespace('wstring_traits')
        self.validate_yes(
            wstring_traits.namespace('yes'),
            declarations.is_std_wstring)
        self.validate_no(
            wstring_traits.namespace('no'),
            declarations.is_std_wstring)


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case
from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):
    COMPILATION_MODE = parser.COMPILATION_MODE.ALL_AT_ONCE

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = 'bit_fields.hpp'
        self.declarations = None

    def setUp(self):
        if not self.declarations:
            self.declarations = parser.parse([self.header], self.config)

    def test(self):
        criteria = declarations.namespace_matcher_t(name='bit_fields')
        declarations.matcher.get_single(criteria, self.declarations)
        self.assertTrue(
            str(criteria) == '(decl type==namespace_t) and (name==bit_fields)')

    def test_allow_empty(self):
        global_ns = declarations.get_global_namespace(self.declarations)
        global_ns.init_optimizer()
        self.assertTrue(
            0 == len(global_ns.namespaces('does not exist', allow_empty=True)))


class unnamed_ns_tester_t(parser_test_case.parser_test_case_t):
    COMPILATION_MODE = parser.COMPILATION_MODE.ALL_AT_ONCE

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = 'unnamed_ns_bug.hpp'
        self.declarations = None

    def setUp(self):
        if not self.declarations:
            self.declarations = parser.parse([self.header], self.config)

    def test(self):
        declarations.matcher.get_single(
            declarations.namespace_matcher_t(name='::'), self.declarations)


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    suite.addTest(unittest.makeSuite(unnamed_ns_tester_t))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

"""
contains classes that allows to extract different information from binary files
( .map, .dll, .so ) and integrate it with existing declarations tree

The main function of this package is
:func:`pygccxml.binary_parsers.parsers.merge_information`.
"""

import warnings
from .undname import undname_creator_t
from .parsers import merge_information


def undecorate_blob(blob):
    """Returns undecorated/unmangled string, created from
    blob(exported symbol name)
    """
    warnings.warn(
        "undecorate_blob is deprecated.\n" +
        "Please have a look at the changelog for an explanation (since 1.8.0)",
        DeprecationWarning)

    return undname_creator_t().undecorate_blob(blob)


def format_decl(decl, hint=None):
    """
    returns string, that represents formatted declaration, according to some
    rules
    :param hint: valid values are: "msvc" and "nm"
    """
    warnings.warn(
        "format_decl is deprecated.\n" +
        "Please have a look at the changelog for an explanation (since 1.8.0)",
        DeprecationWarning)

    return undname_creator_t().format_decl(decl, hint=hint)



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2009 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import unittest
import parser_test_case

from pygccxml import parser
from pygccxml import declarations


class Test(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.header = "test_smart_pointer.hpp"
        self.config.cflags = "-std=c++11"

    def test_smart_pointer(self):
        """
        Test code in the smart_pointer_traits module.

        """

        if self.config.xml_generator == "gccxml":
            return

        decls = parser.parse([self.header], self.config)
        global_ns = declarations.get_global_namespace(decls)

        criteria = declarations.declaration_matcher(name="yes1")
        decls = declarations.matcher.find(criteria, global_ns)
        self.assertTrue(
            declarations.smart_pointer_traits.is_smart_pointer(
                decls[0].decl_type))

        criteria = declarations.declaration_matcher(name="yes2")
        decls = declarations.matcher.find(criteria, global_ns)
        self.assertTrue(
            declarations.auto_ptr_traits.is_smart_pointer(decls[0].decl_type))

        criteria = declarations.declaration_matcher(name="no1")
        decls = declarations.matcher.find(criteria, global_ns)
        self.assertFalse(
            declarations.auto_ptr_traits.is_smart_pointer(decls[0].decl_type))

        criteria = declarations.declaration_matcher(name="no2")
        decls = declarations.matcher.find(criteria, global_ns)
        self.assertFalse(
            declarations.auto_ptr_traits.is_smart_pointer(decls[0].decl_type))


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(Test))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import pprint
import warnings
try:
    import unittest2 as unittest
except ImportError:
    import unittest
import autoconfig
import parser_test_case

import pygccxml
from pygccxml import utils
from pygccxml import parser
from pygccxml import declarations


def is_sub_path(root, some_path):
    root = utils.normalize_path(root)
    some_path = utils.normalize_path(some_path)
    return some_path.startswith(root)


class core_t(parser_test_case.parser_test_case_t):

    """Tests core algorithms of GCC-XML and GCC-XML file reader.
    Those most white-box testing.
    """
    global_ns = None

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.global_ns = None

    def test_top_parent(self):
        enum = self.global_ns.enum('::ns::ns32::E33')
        self.assertTrue(self.global_ns is enum.top_parent)

    # tests namespaces join functionality. described in gccxml.py
    def test_nss_join(self):
        # list of all namespaces
        nss = ['::ns', '::ns::ns12', '::ns::ns22', '::ns::ns32']
        # list of all namespaces that have unnamed namespace
        # unnamed_nss = nss[1:] doing nothing with this list ?
        # list of all enums [0:2] [3:5] [6:8] - has same parent
        enums = [
            '::E11',
            '::E21',
            '::E31',
            '::ns::E12',
            '::ns::E22',
            '::ns::E32',
            '::ns::ns12::E13',
            '::ns::ns22::E23',
            '::ns::ns32::E33']

        for ns in nss:
            self.global_ns.namespace(ns)

        for enum in enums:
            self.global_ns.enum(enum)

        ns = self.global_ns.namespace(nss[0])
        ns12 = self.global_ns.namespace(nss[1])
        ns22 = self.global_ns.namespace(nss[2])
        ns32 = self.global_ns.namespace(nss[3])
        self.assertTrue(
            ns and (
                ns is ns12.parent is ns22.parent is ns32.parent),
            'There are 2 or more instances of ns namespace.')

        e11 = self.global_ns.enum(enums[0])
        e21 = self.global_ns.enum(enums[1])
        e31 = self.global_ns.enum(enums[2])
        self.assertTrue(
            e11.parent is e21.parent is e31.parent,
            'There are 2 or more instances of global namespace.')

        nse12 = self.global_ns.enum(enums[3])
        nse23 = self.global_ns.enum(enums[4])
        nse33 = self.global_ns.enum(enums[5])
        self.assertTrue(
            ns and (
                ns is nse12.parent is nse23.parent is nse33.parent),
            'There are 2 or more instances of ns namespace.')

    def _test_ns_membership(self, ns, enum_name):
        unnamed_enum = ns.enum(
            lambda d: d.name == '' and is_sub_path(
                autoconfig.data_directory,
                d.location.file_name),
            recursive=False)
        self.assertTrue(
            unnamed_enum in ns.declarations,
            "namespace '%s' does not contains unnamed enum." %
            ns.name)

        enum = ns.enum(enum_name, recursive=False)

        self.assertTrue(
            enum in ns.declarations,
            "namespace '%s' does not contains enum '%s'" %
            (ns.name, enum.name))

        self.assertTrue(
            unnamed_enum.parent is ns,
            ("unnamed enum belong to namespace '%s' but this namespace " +
                "is not it's parent.") % ns.name)

        self.assertTrue(
            enum.parent is ns,
            ("enum '%s' belong to namespace '%s' but this namespace" +
                " is not it's parent.") % (enum.name, ns.name))

    def _test_class_membership(self, class_inst, enum_name, access):
        # getting enum through get_members function
        nested_enum1 = class_inst.enum(
            name=enum_name,
            function=declarations.access_type_matcher_t(access))

        # getting enum through declarations property
        nested_enum2 = class_inst.enum(enum_name)

        # it shoud be same object
        self.assertTrue(
            nested_enum1 is nested_enum2,
            ("enum accessed through access definition('%s') and " +
                "through declarations('%s') are different enums " +
                "or instances.") %
            (nested_enum1.name, nested_enum2.name))

        # check whether we meaning same class instance
        self.assertTrue(
            class_inst is nested_enum1.parent is nested_enum2.parent,
            'There are 2 or more instances of ns namespace.')

    # test gccxml_file_reader_t._update_membership algorithm
    def test_membership(self):
        core_membership = self.global_ns.namespace('membership')
        self._test_ns_membership(self.global_ns, 'EGlobal')
        self._test_ns_membership(
            core_membership.namespace('enums_ns'),
            'EWithin')
        self._test_ns_membership(
            core_membership.namespace(''),
            'EWithinUnnamed')
        class_nested_enums = core_membership.class_('class_for_nested_enums_t')
        self._test_class_membership(
            class_nested_enums,
            'ENestedPublic',
            declarations.ACCESS_TYPES.PUBLIC)
        self._test_class_membership(
            class_nested_enums,
            'ENestedProtected',
            declarations.ACCESS_TYPES.PROTECTED)
        self._test_class_membership(
            class_nested_enums,
            'ENestedPrivate',
            declarations.ACCESS_TYPES.PRIVATE)

    def test_compiler_retrocompatibility(self):
        """
        For retro-compatibility, test if the compiler attribute still works.

        This can be removed once the compiler attribute is dropped.

        """

        # Do not clutter the tests with warnings
        warnings.simplefilter("ignore", DeprecationWarning)

        std = self.global_ns.namespace("std")
        self.assertEqual(utils.xml_generator, std.compiler)
        if "GCC" in utils.xml_generator:
            self.assertIn("GCC", std.compiler)
        elif "CastXML" in utils.xml_generator:
            self.assertIn("CastXML", std.compiler)

        # Reset this warning to always
        warnings.simplefilter("error", DeprecationWarning)

    def test_mangled_name_namespace(self):
        std = self.global_ns.namespace("std")
        self.assertTrue(std, "std namespace has not been found")
        # GCCXML had mangled names for everything. With CastXML
        # there are only mangled names for functions and variables.
        if "GCC" in utils.xml_generator:
            self.assertTrue(
                std.mangled,
                "Mangled name of std namespace should be different from None")
        elif "CastXML" in utils.xml_generator:
            # Check if an assertion is correctly raised when using castxml.
            # Call the getter by using lambda, else assertRaises does not
            # work as expected.
            self.assertRaises(Exception, lambda: std.mangled)

    def test_mangled_name_functions(self):
        # This works with gccxml and castxml
        ns = self.global_ns.namespace("overloads")
        do_nothing = ns.calldefs("do_nothing", recursive=False)
        self.assertTrue(
            do_nothing.mangled,
            "Mangled name of do_nothing function should be different +"
            "from None")

    def test_mangled_name_variable(self):
        # This works with gccxml and castxml
        var_inst = self.global_ns.variable('array255')
        self.assertTrue(
            var_inst.mangled,
            "Mangled name of array255 variable should be different +"
            "from None")

    def test_demangled_name_variable(self):
        # This works with gccxml only. Check if an assertion is correctly
        # raised when using castxml.
        var_inst = self.global_ns.variable('array255')
        if "GCC" in utils.xml_generator:
            self.assertTrue(
                var_inst.demangled,
                "Demangled name of array255 variable should be different +"
                "from None")
        elif "CastXML" in utils.xml_generator:
            # Call the getter by using lambda, else assertRaises does not
            # work as expected.
            self.assertRaises(Exception, lambda: var_inst.demangled)

    def _test_is_based_and_derived(self, base, derived, access):
        dhi_v = declarations.hierarchy_info_t(derived, access, True)
        dhi_not_v = declarations.hierarchy_info_t(derived, access, False)
        self.assertTrue(
            dhi_v in base.derived or dhi_not_v in base.derived,
            "base class '%s' doesn't has derived class '%s'" %
            (base.name, derived.name))

        bhi_v = declarations.hierarchy_info_t(base, access, True)
        bhi_not_v = declarations.hierarchy_info_t(base, access, False)

        self.assertTrue(
            bhi_v in derived.bases or bhi_not_v in derived.bases,
            "derive class '%s' doesn't has base class '%s'" %
            (derived.name, base.name))

    def test_class_hierarchy(self):
        class_hierarchy = self.global_ns.namespace('class_hierarchy')

        base = class_hierarchy.class_('base_t')
        other_base = class_hierarchy.class_('other_base_t')
        derived_public = class_hierarchy.class_('derived_public_t')
        derived_protected = class_hierarchy.class_('derived_protected_t')
        derived_private = class_hierarchy.class_('derived_private_t')
        multi_derived = class_hierarchy.class_('multi_derived_t')

        self._test_is_based_and_derived(
            base,
            derived_public,
            declarations.ACCESS_TYPES.PUBLIC)
        self._test_is_based_and_derived(
            base,
            derived_protected,
            declarations.ACCESS_TYPES.PROTECTED)
        self._test_is_based_and_derived(
            base,
            derived_private,
            declarations.ACCESS_TYPES.PRIVATE)
        self._test_is_based_and_derived(
            base,
            multi_derived,
            declarations.ACCESS_TYPES.PROTECTED)
        self._test_is_based_and_derived(
            other_base,
            multi_derived,
            declarations.ACCESS_TYPES.PRIVATE)
        self._test_is_based_and_derived(
            derived_private,
            multi_derived,
            declarations.ACCESS_TYPES.PRIVATE)

    def _test_is_same_bases(self, derived1, derived2):
        bases1 = set([id(hierarchy_info.related_class)
                      for hierarchy_info in derived1.bases])
        bases2 = set([id(hierarchy_info.related_class)
                      for hierarchy_info in derived2.bases])
        self.assertTrue(
            bases1 == bases2,
            ("derived class '%s' and derived class '%s' has references to " +
                "different instance of base classes ") %
            (derived1.name, derived2.name))

    def test_class_join(self):
        diamand_hierarchy = self.global_ns.namespace('diamand_hierarchy')
        base = diamand_hierarchy.class_('base_t')
        derived1 = diamand_hierarchy.class_('derived1_t')
        derived2 = diamand_hierarchy.class_('derived2_t')
        final_derived = diamand_hierarchy.class_('final_derived_t')

        self._test_is_based_and_derived(
            base,
            derived1,
            declarations.ACCESS_TYPES.PUBLIC)
        self._test_is_based_and_derived(
            base,
            derived2,
            declarations.ACCESS_TYPES.PUBLIC)
        self._test_is_based_and_derived(
            derived1,
            final_derived,
            declarations.ACCESS_TYPES.PUBLIC)
        self._test_is_based_and_derived(
            derived2,
            final_derived,
            declarations.ACCESS_TYPES.PUBLIC)
        self._test_is_same_bases(derived1, derived2)

    def test_fundamental_types(self):
        # check whether all build in types could be constructed
        errors = []
        for fundamental_type_name, fundamental_type in \
                declarations.FUNDAMENTAL_TYPES.items():
            if 'complex' in fundamental_type_name:
                continue  # I check this in an other tester
            if isinstance(
                fundamental_type,
                    (declarations.int128_t, declarations.uint128_t)):
                continue  # I don't have test case for this
            if isinstance(fundamental_type, declarations.java_fundamental_t):
                continue  # I don't check this at all
            typedef_name = 'typedef_' + fundamental_type_name.replace(' ', '_')
            typedef = self.global_ns.decl(
                decl_type=declarations.typedef_t,
                name=typedef_name)
            self.assertTrue(
                typedef,
                "unable to find typedef to build-in type '%s'" %
                fundamental_type_name)
            if typedef.decl_type.decl_string != fundamental_type.decl_string:
                errors.append(
                    "there is a difference between typedef base type " +
                    "name('%s') and expected one('%s')" %
                    (typedef.decl_type.decl_string,
                     fundamental_type.decl_string))
        self.assertFalse(errors, pprint.pformat(errors))

    def test_compound_types(self):
        typedef_inst = self.global_ns.decl(
            decl_type=declarations.typedef_t,
            name='typedef_const_int')
        self._test_type_composition(
            typedef_inst.decl_type,
            declarations.const_t,
            declarations.int_t)

        typedef_inst = self.global_ns.decl(
            decl_type=declarations.typedef_t,
            name='typedef_pointer_int')
        self._test_type_composition(
            typedef_inst.decl_type,
            declarations.pointer_t,
            declarations.int_t)

        typedef_inst = self.global_ns.decl(
            decl_type=declarations.typedef_t,
            name='typedef_reference_int')
        self._test_type_composition(
            typedef_inst.decl_type,
            declarations.reference_t,
            declarations.int_t)

        typedef_inst = self.global_ns.decl(
            decl_type=declarations.typedef_t,
            name='typedef_const_unsigned_int_const_pointer')
        self._test_type_composition(
            typedef_inst.decl_type,
            declarations.const_t,
            declarations.pointer_t)
        self._test_type_composition(
            typedef_inst.decl_type.base,
            declarations.pointer_t,
            declarations.const_t)
        self._test_type_composition(
            typedef_inst.decl_type.base.base,
            declarations.const_t,
            declarations.unsigned_int_t)

        typedef_inst = self.global_ns.decl(
            decl_type=declarations.typedef_t,
            name='typedef_volatile_int')
        self._test_type_composition(
            typedef_inst.decl_type,
            declarations.volatile_t,
            declarations.int_t)

        var_inst = self.global_ns.variable('array255')
        self._test_type_composition(
            var_inst.decl_type,
            declarations.array_t,
            declarations.int_t)

        typedef_inst = self.global_ns.decl(
            decl_type=declarations.typedef_t,
            name='typedef_EFavoriteDrinks')
        self.assertTrue(
            isinstance(
                typedef_inst.decl_type,
                declarations.declarated_t),
            " typedef to enum should be 'declarated_t' instead of '%s'" %
            typedef_inst.decl_type.__class__.__name__)
        enum_declaration = self.global_ns.enum('EFavoriteDrinks')
        self.assertTrue(
            typedef_inst.decl_type.declaration is enum_declaration,
            "instance of declaration_t has reference to '%s' instead of '%s'" %
            (typedef_inst.decl_type.declaration.name,
             enum_declaration.name))

    def test_free_function_type(self):
        function_ptr = self.global_ns.decl(
            decl_type=declarations.typedef_t,
            name='function_ptr')
        self._test_type_composition(
            function_ptr.decl_type,
            declarations.pointer_t,
            declarations.free_function_type_t)
        function_type = function_ptr.decl_type.base
        self.assertTrue(
            isinstance(
                function_type.return_type,
                declarations.int_t),
            "return function type of typedef 'function_ptr' should be " +
            "'%s' instead of '%s' " %
            ('int_t', function_type.return_type.__class__.__name__))
        self.assertTrue(
            len(function_type.arguments_types) == 2,
            "number of arguments of function of typedef 'function_ptr' " +
            "should be 2 instead of '%d' " %
            len(function_type.arguments_types))
        self.assertTrue(
            isinstance(
                function_type.arguments_types[0],
                declarations.int_t),
            "first argument of function of typedef 'function_ptr' should be " +
            "'%s' instead of '%s' " %
            ('int_t', function_type.arguments_types[0].__class__.__name__))
        self.assertTrue(
            isinstance(
                function_type.arguments_types[1],
                declarations.double_t),
            "first argument of function of typedef 'function_ptr' should be " +
            "'%s' instead of '%s' " %
            ('double_t', function_type.arguments_types[0].__class__.__name__))

    def test_member_function_type(self):
        function_ptr = self.global_ns.decl(
            decl_type=declarations.typedef_t,
            name='member_function_ptr_t')
        self._test_type_composition(
            function_ptr.decl_type,
            declarations.pointer_t,
            declarations.member_function_type_t)

        function_type = function_ptr.decl_type.base

        members_pointers = self.global_ns.class_('members_pointers_t')
        self.assertTrue(
            function_type.class_inst.declaration is members_pointers,
            "member function type class should be '%s' instead of '%s'" %
            (members_pointers.decl_string,
             function_type.class_inst.decl_string))

        self.assertTrue(
            isinstance(
                function_type.return_type,
                declarations.int_t),
            "return function type of typedef 'member_function_ptr_t' should " +
            "be '%s' instead of '%s' " %
            ('int_t', function_type.return_type.__class__.__name__))
        self.assertTrue(
            len(
                function_type.arguments_types) == 1,
            "number of arguments of function of typedef " +
            "'member_function_ptr_t' should be 1 instead of '%d' " % len(
                function_type.arguments_types))
        self.assertTrue(
            isinstance(
                function_type.arguments_types[0],
                declarations.double_t),
            "first argument of function of typedef 'member_function_ptr_t' " +
            "should be '%s' instead of '%s' " %
            ('double_t', function_type.arguments_types[0].__class__.__name__))

        self.assertTrue(
            function_type.has_const,
            " 'member_function_ptr_t' should be const function.")

    def test_member_variable_type(self):
        mv = self.global_ns.decl(
            decl_type=declarations.typedef_t,
            name='member_variable_ptr_t')
        self._test_type_composition(
            mv.decl_type,
            declarations.pointer_t,
            declarations.member_variable_type_t)

        members_pointers = self.global_ns.class_('members_pointers_t')
        self.assertTrue(
            members_pointers,
            "unable to find class('%s')" %
            'members_pointers_t')
        self._test_type_composition(
            mv.decl_type.base,
            declarations.member_variable_type_t,
            declarations.declarated_t)
        mv_type = mv.decl_type.base
        self.assertTrue(
            mv_type.base.declaration is members_pointers,
            "member function type class should be '%s' instead of '%s'" %
            (members_pointers.decl_string,
             mv_type.base.decl_string))

    def test_overloading(self):
        ns = self.global_ns.namespace('overloads')

        do_nothings = ns.calldefs('do_nothing', recursive=False)
        self.assertTrue(
            4 == len(do_nothings),
            ("expected number of overloaded 'do_nothing' functions is %d " +
                "and existing(%d) is different") %
            (4, len(do_nothings)))
        for index, do_nothing in enumerate(do_nothings):
            others = do_nothings[:index] + do_nothings[index + 1:]
            if set(do_nothing.overloads) != set(others):
                print('\nexisting: ')
                for x in do_nothing.overloads:
                    print(str(x))
                print('\nexpected: ')
                for x in others:
                    print(str(x))

            self.assertTrue(set(do_nothing.overloads) == set(
                others), "there is a difference between expected function " +
                "overloads and existing ones.")

    def test_abstract_classes(self):
        ns = self.global_ns.namespace('abstract_classes')
        abstract_i = ns.class_('abstract_i')
        self.assertTrue(
            abstract_i.is_abstract,
            "class 'abstract_i' should be abstract")
        derived_abstract_i = ns.class_('derived_abstract_i')
        self.assertTrue(
            derived_abstract_i.is_abstract,
            "class 'derived_abstract_i' should be abstract")
        implementation = ns.class_('implementation')
        self.assertTrue(
            not implementation.is_abstract,
            "class 'implementation' should not be abstract")

    def test_versioning(self):
        self.assertTrue(utils.xml_generator)

    def test_byte_size(self):
        mptrs = self.global_ns.class_('members_pointers_t')
        self.assertTrue(mptrs.byte_size != 0)

    def test_byte_align(self):
        mptrs = self.global_ns.class_('members_pointers_t')
        self.assertTrue(mptrs.byte_align != 0)

    def test_byte_offset(self):
        mptrs = self.global_ns.class_('members_pointers_t')
        self.assertTrue(mptrs.variable('xxx').byte_offset != 0)


class core_gccxml_t(core_t):

    """Tests core algorithms of GCC-XML and GCC-XML file reader.
    Those most white-box testing.
    """
    global_ns = None

    def __init__(self, *args):
        core_t.__init__(self, *args)
        self.test_files = [
            'core_ns_join_1.hpp',
            'core_ns_join_2.hpp',
            'core_ns_join_3.hpp',
            'core_membership.hpp',
            'core_class_hierarchy.hpp',
            'core_types.hpp',
            'core_diamand_hierarchy_base.hpp',
            'core_diamand_hierarchy_derived1.hpp',
            'core_diamand_hierarchy_derived2.hpp',
            'core_diamand_hierarchy_final_derived.hpp',
            'core_overloads_1.hpp',
            'core_overloads_2.hpp',
            'abstract_classes.hpp']
        self.global_ns = None

    def setUp(self):
        if not core_t.global_ns:
            decls = parser.parse(
                self.test_files,
                self.config,
                self.COMPILATION_MODE)
            core_t.global_ns = pygccxml.declarations.get_global_namespace(
                decls)
            if self.INIT_OPTIMIZER:
                core_t.global_ns.init_optimizer()
        self.global_ns = core_t.global_ns


class core_all_at_once_t(core_gccxml_t):
    COMPILATION_MODE = parser.COMPILATION_MODE.ALL_AT_ONCE
    INIT_OPTIMIZER = True

    def __init__(self, *args):
        core_gccxml_t.__init__(self, *args)


class core_all_at_once_no_opt_t(core_gccxml_t):
    COMPILATION_MODE = parser.COMPILATION_MODE.ALL_AT_ONCE
    INIT_OPTIMIZER = False

    def __init__(self, *args):
        core_gccxml_t.__init__(self, *args)


class core_file_by_file_t(core_gccxml_t):
    COMPILATION_MODE = parser.COMPILATION_MODE.FILE_BY_FILE
    INIT_OPTIMIZER = True

    def __init__(self, *args):
        core_gccxml_t.__init__(self, *args)


class core_file_by_file_no_opt_t(core_gccxml_t):
    COMPILATION_MODE = parser.COMPILATION_MODE.FILE_BY_FILE
    INIT_OPTIMIZER = False

    def __init__(self, *args):
        core_gccxml_t.__init__(self, *args)


def create_suite():
    suite = unittest.TestSuite()
    if autoconfig.cxx_parsers_cfg.gccxml:
        suite.addTest(unittest.makeSuite(core_all_at_once_t))
        suite.addTest(unittest.makeSuite(core_all_at_once_no_opt_t))
        suite.addTest(unittest.makeSuite(core_file_by_file_t))
        suite.addTest(unittest.makeSuite(core_file_by_file_no_opt_t))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()
# import hotshot
# import hotshot.stats
# statistics_file = tempfile.mkstemp( suffix='.stat' )[1]
# profile = hotshot.Profile(statistics_file)
#
# profile.runcall( run_suite )
# profile.close()
# statistics = hotshot.stats.load( statistics_file )
# statistics.strip_dirs()
# statistics.sort_stats( 'time', 'calls' )
# statistics.print_stats( 678 )



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import os
import unittest
import os.path
import autoconfig
import parser_test_case
from pygccxml.parser.config import xml_generator_configuration_t
from pygccxml.parser import declarations_cache


class decl_cache_tester(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        if not os.path.exists(autoconfig.build_directory):
            os.makedirs(autoconfig.build_directory)

    def test_file_signature(self):
        file1 = os.path.join(autoconfig.data_directory, 'decl_cache_file1.txt')
        file1_dup = os.path.join(
            autoconfig.data_directory,
            'decl_cache_file1_duplicate.txt')
        file2 = os.path.join(autoconfig.data_directory, 'decl_cache_file2.txt')
        sig1 = declarations_cache.file_signature(file1)
        sig1_dup = declarations_cache.file_signature(file1_dup)
        sig2 = declarations_cache.file_signature(file2)
        self.assertTrue(sig1 == sig1_dup)
        self.assertTrue(sig1 != sig2)

    def test_config_signature(self):
        diff_cfg_list = self.build_differing_cfg_list()
        def_cfg = diff_cfg_list[0]
        def_sig = declarations_cache.configuration_signature(def_cfg)

        # Test changes that should cause sig changes
        for cfg in diff_cfg_list[1:]:
            self.assertTrue(
                declarations_cache.configuration_signature(cfg) != def_sig)

        # Test changes that should not cause sig changes
        no_changes = def_cfg.clone()
        self.assertTrue(
            declarations_cache.configuration_signature(no_changes) == def_sig)

        # start_decls_changed = def_cfg.clone()
        # start_decls_changed.start_with_declarations = "test object"
        # self.assertTrue(
        #   configuration_signature(start_decls_changed) == def_sig)

        ignore_changed = def_cfg.clone()
        ignore_changed.ignore_gccxml_output = True
        self.assertTrue(
            declarations_cache.configuration_signature(
                ignore_changed) == def_sig)

    def test_cache_interface(self):
        cache_file = os.path.join(
            autoconfig.build_directory,
            'decl_cache_test.test_cache_read.cache')
        file1 = os.path.join(autoconfig.data_directory, 'decl_cache_file1.txt')
        file1_dup = os.path.join(
            autoconfig.data_directory,
            'decl_cache_file1_duplicate.txt')
        file2 = os.path.join(autoconfig.data_directory, 'decl_cache_file2.txt')
        diff_cfg_list = self.build_differing_cfg_list()
        def_cfg = diff_cfg_list[0]

        if os.path.exists(cache_file):
            os.remove(cache_file)

        cache = declarations_cache.file_cache_t(cache_file)
        self.assertTrue(len(cache._file_cache_t__cache) == 0)

        # test creating new entries for differing files
        cache.update(file1, def_cfg, 1, [])
        self.assertTrue(len(cache._file_cache_t__cache) == 1)
        cache.update(file1_dup, def_cfg, 2, [])
        self.assertTrue(len(cache._file_cache_t__cache) == 1)
        cache.update(file2, def_cfg, 3, [])
        self.assertTrue(len(cache._file_cache_t__cache) == 2)

        self.assertTrue(cache.cached_value(file1, def_cfg) == 2)
        self.assertTrue(cache.cached_value(file2, def_cfg) == 3)

        # Test reading again
        cache.flush()
        cache = declarations_cache.file_cache_t(cache_file)
        self.assertTrue(len(cache._file_cache_t__cache) == 2)
        self.assertTrue(cache.cached_value(file1, def_cfg) == 2)
        self.assertTrue(cache.cached_value(file2, def_cfg) == 3)

        # Test flushing doesn't happen if we don't touch the cache
        cache = declarations_cache.file_cache_t(cache_file)
        self.assertTrue(
            cache.cached_value(
                file1, def_cfg) == 2)  # Read from cache
        cache.flush()    # should not actually flush
        cache = declarations_cache.file_cache_t(cache_file)
        self.assertTrue(len(cache._file_cache_t__cache) == 2)

        # Test flush culling
        cache = declarations_cache.file_cache_t(cache_file)
        cache.update(file1_dup, def_cfg, 4, [])    # Modify cache
        cache.flush()    # should cull off one entry
        cache = declarations_cache.file_cache_t(cache_file)
        self.assertTrue(len(cache._file_cache_t__cache) == 1)

    @staticmethod
    def build_differing_cfg_list():
        """ Return a list of configurations that all differ. """
        cfg_list = []
        def_cfg = xml_generator_configuration_t(
            "xml_generator_path",
            '.', ['tmp'], ['sym'], ['unsym'], None, False, "")
        cfg_list.append(def_cfg)

        # Test changes that should cause sig changes
        gccxml_changed = def_cfg.clone()
        gccxml_changed.xml_generator_path = "other_path"
        cfg_list.append(gccxml_changed)

        wd_changed = def_cfg.clone()
        wd_changed.working_directory = "other_dir"
        cfg_list.append(wd_changed)

        # inc_changed = def_cfg.clone()
        # inc_changed.include_paths = ["/var/tmp"]
        # self.assertTrue(configuration_signature(inc_changed) != def_sig)
        inc_changed = xml_generator_configuration_t(
            "xml_generator_path", '.', ['/var/tmp'], ['sym'], ['unsym'],
            None, False, "")
        cfg_list.append(inc_changed)

        # def_changed = def_cfg.clone()
        # def_changed.define_symbols = ["symbol"]
        # self.assertTrue(configuration_signature(def_changed) != def_sig)
        def_changed = xml_generator_configuration_t(
            "xml_generator_path", '.', ['/var/tmp'], ['new-sym'], ['unsym'],
            None, False, "")
        cfg_list.append(def_changed)

        # undef_changed = def_cfg.clone()
        # undef_changed.undefine_symbols = ["symbol"]
        # self.assertTrue(configuration_signature(undef_changed) != def_sig)
        undef_changed = xml_generator_configuration_t(
            "xml_generator_path", '.', ['/var/tmp'], ['sym'], ['new-unsym'],
            None, False, "")
        cfg_list.append(undef_changed)

        cflags_changed = def_cfg.clone()
        cflags_changed.cflags = "new flags"
        cfg_list.append(cflags_changed)

        return cfg_list


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(decl_cache_tester))
    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()



# Copyright 2014-2016 Insight Software Consortium.
# Copyright 2004-2008 Roman Yakovenko.
# Distributed under the Boost Software License, Version 1.0.
# See http://www.boost.org/LICENSE_1_0.txt

import pprint
import unittest
import autoconfig
import parser_test_case

from pygccxml import parser
from pygccxml import declarations
from pygccxml import utils


class declarations_t(parser_test_case.parser_test_case_t):

    def __init__(self, *args):
        parser_test_case.parser_test_case_t.__init__(self, *args)
        self.global_ns = None

    def test_enumeration_t(self):
        enum = self.global_ns.enum('ENumbers')
        expected_values = list(
            zip(['e%d' % index for index in range(10)],
                [index for index in range(10)]))
        self.assertTrue(
            expected_values == enum.values,
            ("expected enum values ( '%s' ) and existings ( '%s' ) are " +
                "different") %
            (pprint.pformat(expected_values), pprint.pformat(enum.values)))

    def test_namespace(self):
        pass  # tested in core_tester

    def test_types(self):
        pass  # tested in core_tester

    def test_variables(self):
        self.global_ns.namespace('variables')
        initialized = self.global_ns.variable(name='initialized')

        expected_value = None
        if '0.9' in utils.xml_generator:
            expected_value = '10122004ul'
        else:
            expected_value = '10122004'

        self.assertTrue(
            initialized.value == expected_value,
            ("there is a difference between expected value( %s ) and real " +
                "value(%s) of 'initialized' variable") %
            (expected_value, initialized.value))
        self._test_type_composition(
            initialized.decl_type,
            declarations.const_t,
            declarations.long_unsigned_int_t)

        static_var = initialized = self.global_ns.variable(name='static_var')
        self.assertTrue(
            static_var.type_qualifiers.has_static,
            "static_var must have static type qualifier")
        self.assertTrue(
            not static_var.type_qualifiers.has_mutable,
            "static_var must not have mutable type qualifier")

        if 'PDB' in utils.xml_generator:
            return  # TODO find out work around

        m_mutable = initialized = self.global_ns.variable(name='m_mutable')
        self.assertTrue(
            not m_mutable.type_qualifiers.has_static,
            "m_mutable must not have static type qualifier")
        # TODO: "There is bug in GCCXML: doesn't write mutable qualifier."
        # self.assertTrue( m_mutable.type_qualifiers.has_mutable
        #                 , "static_var must have mutable type qualifier" )

    def test_calldef_free_functions(self):
        ns = self.global_ns.namespace('calldef')

        no_return_no_args = ns.free_function('no_return_no_args')

        self._test_calldef_return_type(no_return_no_args, declarations.void_t)

        # TODO: gccxml reported no_return_no_args as having an extern
        # qualifier, which is wrong; Keep the test like this for gccxml as
        # gccxml will be dropped one day. With castxml check if function has
        # no extern qualifier.
        if 'GCC-XML' in utils.xml_generator:
            self.assertTrue(
                no_return_no_args.has_extern,
                "function 'no_return_no_args' should have an extern qualifier")
        else:
            self.assertTrue(
                not no_return_no_args.has_extern,
                "function 'no_return_no_args' should have an extern qualifier")

        # Static_call is explicetely defined as extern, this works with gccxml
        # and castxml.
        static_call = ns.free_function('static_call')
        self.assertTrue(
            static_call,
            "function 'no_return_no_args' should have an extern qualifier")

        return_no_args = ns.free_function('return_no_args')
        self._test_calldef_return_type(return_no_args, declarations.int_t)
        # from now there is no need to check return type.
        no_return_1_arg = ns.free_function(name='no_return_1_arg')
        self.assertTrue(
            no_return_1_arg,
            "unable to find 'no_return_1_arg' function")
        self.assertTrue(no_return_1_arg.arguments[0].name in ['arg', 'arg0'])
        self._test_calldef_args(
            no_return_1_arg,
            [declarations.argument_t(
                name=no_return_1_arg.arguments[0].name,
                decl_type=declarations.int_t())])

        return_default_args = ns.free_function('return_default_args')
        self.assertTrue(
            return_default_args.arguments[0].name in ['arg', 'arg0'])
        self.assertTrue(
            return_default_args.arguments[1].name in ['arg1', 'flag'])
        self._test_calldef_args(
            return_default_args,
            [declarations.argument_t(
                name=return_default_args.arguments[0].name,
                decl_type=declarations.int_t(),
                default_value='1'),
                declarations.argument_t(
                    name=return_default_args.arguments[1].name,
                    decl_type=declarations.bool_t(),
                    default_value='false')])
        self._test_calldef_exceptions(return_default_args, [])

        calldef_with_throw = ns.free_function('calldef_with_throw')
        self.assertTrue(
            calldef_with_throw,
            "unable to find 'calldef_with_throw' function")
        self._test_calldef_exceptions(
            calldef_with_throw, [
                'some_exception_t', 'other_exception_t'])
        # from now there is no need to check exception specification

    def test_calldef_member_functions(self):
        struct_calldefs = self.global_ns.class_('calldefs_t')

        member_inline_call = struct_calldefs.member_function(
            'member_inline_call')
        self._test_calldef_args(
            member_inline_call, [
                declarations.argument_t(
                    name='i', decl_type=declarations.int_t())])

        member_const_call = struct_calldefs.member_function(
            'member_const_call')
        self.assertTrue(
            member_const_call.has_const,
            "function 'member_const_call' should have const qualifier")
        self.assertTrue(
            member_const_call.virtuality ==
            declarations.VIRTUALITY_TYPES.NOT_VIRTUAL,
            "function 'member_const_call' should be non virtual function")

        member_virtual_call = struct_calldefs.member_function(
            name='member_virtual_call')
        self.assertTrue(
            member_virtual_call.virtuality ==
            declarations.VIRTUALITY_TYPES.VIRTUAL,
            "function 'member_virtual_call' should be virtual function")

        member_pure_virtual_call = struct_calldefs.member_function(
            'member_pure_virtual_call')
        self.assertTrue(
            member_pure_virtual_call.virtuality ==
            declarations.VIRTUALITY_TYPES.PURE_VIRTUAL,
            ("function 'member_pure_virtual_call' should be pure virtual " +
                "function"))

        static_call = struct_calldefs.member_function('static_call')
        self.assertTrue(
            static_call.has_static,
            "function 'static_call' should have static qualifier")
        # from now we there is no need to check static qualifier

    def test_constructors_destructors(self):
        struct_calldefs = self.global_ns.class_('calldefs_t')

        destructor = struct_calldefs.calldef('~calldefs_t')
        self._test_calldef_args(destructor, [])
        self._test_calldef_return_type(destructor, None.__class__)

        # well, now we have a few functions ( constructors ) with the same
        # name, there is no easy way to find the desired one. Well in my case
        # I have only 4 constructors
        # 1. from char
        # 2. from (int,double)
        # 3. default
        # 4. copy constructor
        constructor_found = struct_calldefs.constructors('calldefs_t')
        self.assertTrue(
            len(constructor_found) == 5,
            ("struct 'calldefs_t' has 5 constructors, pygccxml parser " +
                "reports only about %d.") %
            len(constructor_found))
        error_text = "copy constructor has not been found"
        self.assertTrue(1 == len(
            [constructor for constructor in constructor_found if
                declarations.is_copy_constructor(constructor)]), error_text)
        # there is nothing to check about constructors - I know the
        # implementation of parser.
        # In this case it doesn't different from any other function

        c = struct_calldefs.constructor('calldefs_t', arg_types=['char'])
        self.assertTrue(
            c.explicit,
            ("calldef_t constructor defined with 'explicit' keyword, " +
                "for some reason the value is False "))

        arg_type = declarations.declarated_t(
            self.global_ns.class_('some_exception_t'))
        c = struct_calldefs.constructor('calldefs_t', arg_types=[arg_type])
        self.assertTrue(
            c.explicit is False,
            ("calldef_t constructor defined without 'explicit' keyword, " +
                "for some reason the value is True "))

    def test_operator_symbol(self):
        calldefs_operators = ['=', '==']
        calldefs_cast_operators = ['char *', 'double']
        struct_calldefs = self.global_ns.class_('calldefs_t')
        self.assertTrue(struct_calldefs, "unable to find struct 'calldefs_t'")
        for decl in struct_calldefs.declarations:
            if not isinstance(decl, declarations.operator_t):
                continue
            if not isinstance(decl, declarations.casting_operator_t):
                self.assertTrue(
                    decl.symbol in calldefs_operators,
                    "unable to find operator symbol for operator '%s'" %
                    decl.decl_string)
            else:
                self.assertTrue(
                    decl.return_type.decl_string in calldefs_cast_operators,
                    "unable to find operator symbol for operator '%s'" %
                    decl.decl_string)

    def test_ellipsis(self):
        ns = self.global_ns.namespace('ellipsis_tester')
        do_smth = ns.mem_fun('do_smth')
        self.assertTrue(do_smth.has_ellipsis)
        do_smth_else = ns.free_fun('do_smth_else')
        self.assertTrue(do_smth_else.has_ellipsis)


class gccxml_declarations_t(declarations_t):
    global_ns = None

    def __init__(self, *args):
        declarations_t.__init__(self, *args)
        self.test_files = [
            'declarations_enums.hpp',
            'declarations_variables.hpp',
            'declarations_calldef.hpp']
        self.global_ns = None

    def setUp(self):
        if not gccxml_declarations_t.global_ns:
            decls = parser.parse(
                self.test_files,
                self.config,
                self.COMPILATION_MODE)
            gccxml_declarations_t.global_ns = \
                declarations.get_global_namespace(decls)
        if not self.global_ns:
            self.global_ns = gccxml_declarations_t.global_ns


class all_at_once_tester_t(gccxml_declarations_t):
    COMPILATION_MODE = parser.COMPILATION_MODE.ALL_AT_ONCE

    def __init__(self, *args):
        gccxml_declarations_t.__init__(self, *args)


class file_by_file_tester_t(gccxml_declarations_t):
    COMPILATION_MODE = parser.COMPILATION_MODE.FILE_BY_FILE

    def __init__(self, *args):
        gccxml_declarations_t.__init__(self, *args)


class pdb_based_tester_t(declarations_t):

    def __init__(self, *args):
        declarations_t.__init__(self, *args)
        self.global_ns = autoconfig.get_pdb_global_ns()


def create_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(file_by_file_tester_t))
    suite.addTest(unittest.makeSuite(all_at_once_tester_t))
    # if os.name == 'nt' and autoconfig.get_pdb_global_ns():
    # suite.addTest( unittest.makeSuite(pdb_based_tester_t))

    return suite


def run_suite():
    unittest.TextTestRunner(verbosity=2).run(create_suite())

if __name__ == "__main__":
    run_suite()
